import binoculars.util
import os

import unittest


class TestCase(unittest.TestCase):
    def setUp(self):
        fn = "examples/configs/example_config_id03"
        self.cfg = binoculars.util.ConfigFile.fromtxtfile(fn)

    def test_IO(self):
        self.cfg.totxtfile("test.txt")
        self.cfg.tofile("test.hdf5")
        print(binoculars.util.ConfigFile.fromfile("test.hdf5"))
        self.assertRaises(IOError, binoculars.util.ConfigFile.fromtxtfile, "")
        self.assertRaises(IOError, binoculars.util.ConfigFile.fromfile, "")

    def tearDown(self):
        os.remove("test.txt")
        os.remove("test.hdf5")


if __name__ == "__main__":
    unittest.main()



"""
BINocular backend for beamline BM25, branch B first endstation [1] 
This backend should serve as a basic implementation of a backend based on
xrayutilities [2]. It uses the information from the edf files (motors position
and detector image) ignoring the spec file, except for using its scan numbers
to identify images belonging to the same scan.

You should use CCD file names generated with the following pattern:
filename_#n_#p_#r.edf  (n: spec-scan number, p: point number, r: image number)
Binning (2,2)

The backend is called 'EH2SCD'.

Created on 2014-10-28

[1] http://www.esrf.eu/UsersAndScience/Experiments/CRG/BM25/BeamLine/experimentalstations/Single_Crystal_Diffraction
[2] http://xrayutilities.sourceforge.net/

author: Dominik Kriegner (dominik.kriegner@gmail.com)
"""

import sys
import os
import glob
import numpy
import xrayutilities as xu

from .. import backend, errors, util


class HKLProjection(backend.ProjectionBase):
    # scalars: mu, theta, phi, chi, ccdty, ccdtx, ccdtz, ccdth, wavelength
    # 3x3 matrix: UB
    def project(
        self,
        mu,
        theta,
        phi,
        chi,
        ccdty,
        ccdtx,
        ccdtz,
        ccdth,
        ccdtr,
        wavelength,
        UB,
        qconv,
    ):
        qconv.wavelength = wavelength
        h, k, l = qconv.area(
            mu, theta, phi, chi, ccdty, ccdtx, ccdtz, ccdth, UB=UB.reshape((3, 3))
        )
        return (h, k, l)

    def get_axis_labels(self):
        return "H", "K", "L"


class HKProjection(HKLProjection):
    def project(
        self,
        mu,
        theta,
        phi,
        chi,
        ccdty,
        ccdtx,
        ccdtz,
        ccdth,
        ccdtr,
        wavelength,
        UB,
        qconv,
    ):
        H, K, L = super(HKProjection, self).project(
            mu,
            theta,
            phi,
            chi,
            ccdty,
            ccdtx,
            ccdtz,
            ccdth,
            ccdtr,
            wavelength,
            UB,
            qconv,
        )
        return (H, K)

    def get_axis_labels(self):
        return "H", "K"


class QProjection(backend.ProjectionBase):
    def project(
        self,
        mu,
        theta,
        phi,
        chi,
        ccdty,
        ccdtx,
        ccdtz,
        ccdth,
        ccdtr,
        wavelength,
        UB,
        qconv,
    ):
        qconv.wavelength = wavelength
        qx, qy, qz = qconv.area(
            mu, theta, phi, chi, ccdty, ccdtx, ccdtz, ccdth, ccdtr, UB=numpy.identity(3)
        )
        return (qx, qy, qz)

    def get_axis_labels(self):
        return "qx", "qy", "qz"


class QinpProjection(backend.ProjectionBase):
    def project(
        self,
        mu,
        theta,
        phi,
        chi,
        ccdty,
        ccdtx,
        ccdtz,
        ccdth,
        ccdtr,
        wavelength,
        UB,
        qconv,
    ):
        qconv.wavelength = wavelength
        qx, qy, qz = qconv.area(
            mu, theta, phi, chi, ccdty, ccdtx, ccdtz, ccdth, ccdtr, UB=numpy.identity(3)
        )
        return (numpy.sqrt(qx ** 2 + qy ** 2), qz)

    def get_axis_labels(self):
        return "qinp", "qz"


class EDFInput(backend.InputBase):
    # OFFICIAL API
    def generate_jobs(self, command):
        scans = util.parse_multi_range(",".join(command).replace(" ", ","))
        imgs = self.list_images(scans)
        imgcount = len(imgs)
        if not len(imgs):
            sys.stderr.write("error: no images selected, nothing to do\n")
        # next(self.get_images(imgs, 0, imgcount-1, dry_run=True))# dryrun

        for s in util.chunk_slicer(imgcount, self.config.target_weight):
            yield backend.Job(
                images=imgs,
                firstimage=s.start,
                lastimage=s.stop - 1,
                weight=s.stop - s.start,
            )

    def process_job(self, job):
        super(EDFInput, self).process_job(job)
        images = self.get_images(job.images, job.firstimage, job.lastimage)  # iterator!

        for image in images:
            yield self.process_image(image)

    def parse_config(self, config):
        super(EDFInput, self).parse_config(config)
        self.config.xmask = util.parse_multi_range(config.pop("xmask"))
        self.config.ymask = util.parse_multi_range(config.pop("ymask"))
        self.config.imagefile = config.pop("imagefile")
        self.config.UB = config.pop("ub", None)
        if self.config.UB:
            self.config.UB = util.parse_tuple(self.config.UB, length=9, type=float)
        self.config.sddx = float(config.pop("sddx_offset"))
        self.config.sddy = float(config.pop("sddy_offset"))
        self.config.sddz = float(config.pop("sddz_offset"))
        self.config.ccdth0 = float(config.pop("ccdth_offset"))
        self.config.pixelsize = util.parse_tuple(
            config.pop("pixelsize"), length=2, type=float
        )
        self.config.centralpixel = util.parse_tuple(
            config.pop("centralpixel"), length=2, type=float
        )

    def get_destination_options(self, command):
        if not command:
            return False
        command = ",".join(command).replace(" ", ",")
        scans = util.parse_multi_range(command)
        return dict(first=min(scans), last=max(scans), range=",".join(command))

    # CONVENIENCE FUNCTIONS
    @staticmethod
    def apply_mask(data, xmask, ymask):
        roi = data[ymask, :]
        return roi[:, xmask]

    # MAIN LOGIC
    def list_images(self, scannrs):
        pattern = self.config.imagefile
        imgfiles = []
        # check if necessary image-files exist
        for nr in scannrs:
            try:
                fpattern = pattern.format(scannr=nr)
            except Exception as e:
                raise errors.ConfigError(
                    "invalid 'imagefile' specification '{0}': {1}".format(
                        self.config.imagefile, e
                    )
                )

            files = glob.glob(fpattern)
            if len(files) == 0:
                raise errors.FileError(
                    "needed file do not exist: scannr {0}".format(nr)
                )
            else:
                imgfiles += files
        return imgfiles

    def get_images(self, imgs, first, last, dry_run=False):
        for i in range(first, last + 1):
            img = imgs[i]
            if dry_run:
                yield
            else:
                edf = xu.io.EDFFile(img)
                yield edf


class EH2SCD(EDFInput):
    monitor_counter = "C_mont"
    # define BM25 goniometer, SIXC geometry? with 2D detector mounted on
    # translation-axes
    # see http://www.esrf.eu/UsersAndScience/Experiments/CRG/BM25/BeamLine/experimentalstations/Single_Crystal_Diffraction
    # The geometry is: 4S + translations and one det. rotation
    # sample axis: mu, th, chi, phi
    # detector axis: translations + theta rotation (to make beam perpendicular
    #                to the detector plane in symmetric arrangement)
    qconv = xu.experiment.QConversion(
        ["x+", "z+", "y+", "x+"], ["ty", "tx", "tz", "x+", "ty"], [0, 1, 0]
    )
    # convention for coordinate system: y downstream; x in bound; z upwards
    # (righthanded)
    # QConversion will set up the goniometer geometry. So the first argument
    # describes the sample rotations, the second the detector rotations and the
    # third the primary beam direction.

    def parse_config(self, config):
        super(EH2SCD, self).parse_config(config)
        centralpixel = self.config.centralpixel
        # define detector parameters
        roi = (
            self.config.ymask[0],
            self.config.ymask[-1] + 1,
            self.config.xmask[0],
            self.config.xmask[-1] + 1,
        )
        self.qconv.init_area(
            "z-",
            "x+",
            cch1=centralpixel[1],
            cch2=centralpixel[0],
            Nch1=1912,
            Nch2=3825,
            pwidth1=self.config.pixelsize[1],
            pwidth2=self.config.pixelsize[0],
            distance=1e-10,
            roi=roi,
        )
        print(
            (
                "{:>20} {:>9} {:>10} {:>9} {:>9} {:>9}".format(
                    " ", "Mu", "Theta", "CCD_Y", "CCD_X", "CCD_Z"
                )
            )
        )

    def process_image(self, image):
        # motor positions
        mu = float(image.header["M_mu"])
        th = float(image.header["M_th"])
        chi = float(image.header["M_chi"])
        phi = float(image.header["M_phi"])
        # distance 'ctr' corresponds to distance of the detector chip from
        # the CCD_TH rotation axis. The rest is handled by the translations
        ctr = -270.0  # measured by ruler only!!!
        cty = float(image.header["M_CCD_Y"]) - self.config.sddy - ctr
        ctx = float(image.header["M_CCD_X"]) - self.config.sddx
        ctz = float(image.header["M_CCD_Z"]) - self.config.sddz
        cth = float(image.header["M_CCD_TH"]) - self.config.ccdth0

        # filter correction
        transm = 1.0  # no filter correction! (Filters are manual on BM25!)

        mon = float(image.header[self.monitor_counter])
        wavelength = float(image.header["WAVELENGTH"])
        if self.config.UB:
            UB = self.config.UB
        else:
            UB = self._get_UB(image.header)

        # normalization
        data = image.data / mon / transm
        print(
            (
                "{:>20} {:9.4f} {:10.4f} {:9.1f} {:9.1f} {:9.1f}".format(
                    os.path.split(image.filename)[-1], mu, th, cty, ctx, ctz
                )
            )
        )

        # masking
        intensity = self.apply_mask(data, self.config.xmask, self.config.ymask)

        return (
            intensity,
            numpy.ones_like(intensity),
            (
                mu,
                th,
                phi,
                chi,
                cty,
                ctx,
                ctz,
                cth,
                ctr,  ## weights added to API. Treated here like before
                wavelength,
                UB,
                self.qconv,
            ),
        )

    @staticmethod
    def _get_UB(header):
        ub = numpy.zeros(9)
        for i in range(9):
            ub[i] = float(header["UB{:d}".format(i)])
        return ub



# -*- coding: utf-8 -*-
"""
Created on Tue Feb 18 09:44:47 2020

@author: Prevot
"""
# we use x=np.nonzero(contributions) -> for example x[0]=h values ,x[1]=k values

import numpy as np
from os import path, getcwd

from lmfit import Parameters
from guidata.qt.QtCore import Qt
from guidata.qt.QtGui import (
    QListWidget,
    QListWidgetItem,
    QTableWidget,
    QTableWidgetItem,
    QLabel,
    QMenu,
    QAction,
    QCursor,
    QPixmap,
    QMessageBox,
    QFileDialog,
    QMainWindow,
    QSplitter,
    QWidget,
    QComboBox,
    QVBoxLayout,
    QHBoxLayout,
)
from guiqwt.builder import PlotItemBuilder as GuiPlotItemBuilder
from guiqwt.config import _
from guidata.configtools import add_image_path
from guiqwt.curve import CurvePlot
from guiqwt.histogram import ContrastAdjustment, lut_range_threshold
from guiqwt.interfaces import IPanel
from guiqwt.image import ImagePlot, ImageItem
from guiqwt.plot import PlotManager
from guiqwt._scaler import _histogram
from guiqwt.styles import ImageParam

from grafit.calculation import get_xywxwy_cxy
from grafit.signals import (
    SIG_TRY_CHANGED,
    SIG_TRIGGERED,
    SIG_MODEL_ADDED,
    SIG_MODEL_REMOVED,
)
from grafit.tools import (
    ShowWeightsTool,
    RunTool,
    SaveFitTool,
    PrefTool,
    CircularActionToolCXY,
    AddModelTool,
    MultiPointTool,
)
from grafit.models import list_of_2D_models

abspath = path.abspath(__file__)
dirpath = path.dirname(abspath)

# we add the current directory to have access to the images
add_image_path(dirpath)


ID_PARAMETER = "parameters"

minimizationmethods = [
    ("leastsq", "Levenberg-Marquardt"),
    ("least_squares", "Least-Squares"),
    ("differential_evolution", "Differential evolution"),
    ("brute", "Brute force"),
    ("nelder", "Nelder-Mead"),
    ("lbfgsb", "L-BFGS-B"),
    ("powell", "Powell"),
    # ('cg','Conjugate-Gradient'),   :Jacobian is required
    # ('newton','Newton-Congugate-Gradient'), Jacobian is required
    ("cobyla", "Cobyla"),
    # ('tnc','Truncate Newton'),Jacobian is required
    # ('trust-ncg','Trust Newton-Congugate-Gradient'),Jacobian is required
    # ('dogleg','Dogleg'),Jacobian is required
    ("basinhopping", "Basin-hopping"),
    ("slsqp", "Sequential Linear Squares Programming"),
]


def _nanmin(data):
    if data.dtype.name in ("float32", "float64", "float128"):
        return np.nanmin(data)
    else:
        return data.min()


def _nanmax(data):
    if data.dtype.name in ("float32", "float64", "float128"):
        return np.nanmax(data)
    else:
        return data.max()


class ImageNan(ImageItem):
    def auto_lut_scale(self):
        _min, _max = _nanmin(self.data), _nanmax(self.data)
        self.set_lut_range([_min, _max])

    def auto_lut_scale_sym(self):
        _max = max(abs(_nanmin(self.data)), abs(_nanmax(self.data)))
        self.set_lut_range([-_max, _max])

    def get_histogram(self, nbins):
        """interface de IHistDataSource"""
        if self.data is None:
            return [0,], [0, 1]
        if self.histogram_cache is None or nbins != self.histogram_cache[0].shape[0]:
            # from guidata.utils import tic, toc
            if False:
                # tic("histo1")
                res = np.histogram(self.data[~np.isnan(self.data)], nbins)
                # toc("histo1")
            else:
                # TODO: _histogram is faster, but caching is buggy
                # in this version
                # tic("histo2")
                _min = _nanmin(self.data)
                _max = _nanmax(self.data)
                if self.data.dtype in (np.float64, np.float32):
                    bins = np.unique(
                        np.array(
                            np.linspace(_min, _max, nbins + 1), dtype=self.data.dtype
                        )
                    )
                else:
                    bins = np.arange(_min, _max + 2, dtype=self.data.dtype)
                res2 = np.zeros((bins.size + 1,), np.uint32)
                _histogram(self.data.flatten(), bins, res2)
                # toc("histo2")
                res = res2[1:-1], bins
            self.histogram_cache = res
        else:
            res = self.histogram_cache
        return res


class PlotItemBuilder(GuiPlotItemBuilder):
    def __init__(self):
        super(PlotItemBuilder, self).__init__()

    def imagenan(
        self,
        data=None,
        filename=None,
        title=None,
        alpha_mask=None,
        alpha=None,
        background_color=None,
        colormap=None,
        xdata=[None, None],
        ydata=[None, None],
        pixel_size=None,
        center_on=None,
        interpolation="linear",
        eliminate_outliers=None,
        xformat="%.1f",
        yformat="%.1f",
        zformat="%.1f",
    ):
        """
        Make an image `plot item` from data
        (:py:class:`guiqwt.image.ImageItem` object or
        :py:class:`guiqwt.image.RGBImageItem` object if data has 3 dimensions)
        """
        assert isinstance(xdata, (tuple, list)) and len(xdata) == 2
        assert isinstance(ydata, (tuple, list)) and len(ydata) == 2
        param = ImageParam(title="Image", icon="image.png")
        data, filename, title = self._get_image_data(
            data, filename, title, to_grayscale=True
        )
        assert data.ndim == 2, "Data must have 2 dimensions"
        if pixel_size is None:
            assert center_on is None, (
                "Ambiguous parameters: both `center_on`"
                " and `xdata`/`ydata` were specified"
            )
            xmin, xmax = xdata
            ymin, ymax = ydata
        else:
            xmin, xmax, ymin, ymax = self.compute_bounds(data, pixel_size, center_on)
        self.__set_image_param(
            param,
            title,
            alpha_mask,
            alpha,
            interpolation,
            background=background_color,
            colormap=colormap,
            xmin=xmin,
            xmax=xmax,
            ymin=ymin,
            ymax=ymax,
            xformat=xformat,
            yformat=yformat,
            zformat=zformat,
        )

        image = ImageNan(data, param)
        image.set_filename(filename)
        if eliminate_outliers is not None:
            image.set_lut_range(lut_range_threshold(image, 256, eliminate_outliers))
        return image


make = PlotItemBuilder()


class ExtendedParams:
    # dictionnaire de la liste des courbes pour le fit
    # utilise pour comprendre les Parameters de lmfit
    # ne pilote jamais le tracage des courbes et l'ecriture dans le tableau
    def __init__(self):
        self.reset()

    def reset(self):
        self.modelclasses = (
            list()
        )  # list of the Model classes that compose the Model for fitting the data
        self.models = (
            list()
        )  # list of the Model instances that compose the Model for fitting the data
        self.composite_model = None
        self.ncv = len(self.models)
        self.partry = Parameters()  # parametres d'essai
        self.paropt = Parameters()  # parametres optimises
        self.tags = []
        self.method = "leastsq"
        self.chi2opt = None
        self.dataimage = None  # associated data to be fitted
        self.isfitted = False  # is the data fitted?
        self.issaved = False  # has the fit result been saved?
        self.tags = []  # tags is a list of (name,value)

    def printparams(self):
        print("printparams")
        for key in self.partry.keys():
            print(key, self.partry[key].value, self.partry[key].vary)

    def make_composite_model(self, partry=True):
        # return a composite model based on the list of modelclasses
        # restart the building of models and make new try parameters if partry==True
        self.models = []
        if len(self.modelclasses) == 0:
            self.composite_model = None
            return None
        else:
            model = self.modelclasses[0](prefix="cv0_")
            self.models.append(model)
            self.composite_model = self.models[0]
            if len(self.modelclasses) > 2:
                for k, modelclass in enumerate(self.modelclasses[1:], 1):
                    model = self.modelclasses[0](prefix="cv%d_" % k)
                    self.models.append(model)
                    self.composite_model = self.composite_model + model
        if partry:
            self.partry = self.composite_model.make_params()
            self.update_tags(self.tags)

    def add_model_class(self, modelclass, paramvalues=None):
        # ajoute une courbe dans la liste parametres
        self.modelclasses.append(modelclass)
        k = len(self.modelclasses) - 1
        model = modelclass(prefix="cv%d_" % k)
        self.models.append(model)
        if k > 0:  # there is already a model
            self.composite_model = self.composite_model + model
        else:
            self.composite_model = model

        self.partry = self.composite_model.make_params()
        self.update_tags(self.tags)
        return model

    def remove_model_class(self, k):
        correspondance = range(k) + [-1] + range(k, len(self.modelclasses) - 1)
        self.modelclasses.pop(k)
        self.make_composite_model(partry=False)
        self.rebuild_partry(correspondance)

    def rebuild_partry(self, correspondance):
        # rebuild a set of try parameters from correspondance between indices
        # correspondance is the list of old indices pointing toward new ones or nothing if equal to -1

        # first we set values and restrains
        parameters = Parameters()
        for old_entry in self.partry:
            k = self.cvnumber(old_entry)
            if k is not None:  # this is a curve
                if correspondance[k] >= 0:  # otherwise the curve has been suppressed
                    old_txt = "cv%d" % k
                    new_txt = "cv%d" % correspondance[k]
                    new_entry = old_entry.replace(old_txt, new_txt)
                    parameters.add(
                        new_entry,
                        value=self.partry[old_entry].value,
                        vary=self.partry[old_entry].vary,
                        min=self.partry[old_entry].min,
                        max=self.partry[old_entry].max,
                    )
            else:  # this is a fixed parameter (tag)
                parameters.add(
                    old_entry,
                    value=self.partry[old_entry].value,
                    vary=self.partry[old_entry].vary,
                    min=self.partry[old_entry].min,
                    max=self.partry[old_entry].max,
                )

        # then we set expressions that also have to be converted
        for old_entry in self.partry:
            j = self.cvnumber(old_entry)
            if j is not None:  # this is a curve
                if correspondance[j] >= 0:  # otherwise the curve has been suppressed
                    old_txt = "cv%d" % j
                    new_txt = "cv%d" % correspondance[j]
                    new_entry = old_entry.replace(old_txt, new_txt)  # new entry
                    new_expr = self.partry[old_entry].expr  # start with old expression
                    if new_expr is not None and len(new_expr) > 0:
                        for k in correspondance:
                            old_txt = "cv%d" % k
                            if old_txt in new_expr:
                                if correspondance[k] == -1:
                                    # the curve referenced in expression has been removed! we cannot maintain the expression
                                    new_expr = ""
                                else:
                                    new_txt = "cv%d" % correspondance[k]
                                    new_expr = new_expr.replace(old_txt, new_txt)
                        parameters[new_entry].set(expr=new_expr)
        self.partry = parameters
        self.update_tags(self.tags)

    def set_dataimage(self, dataimage):
        self.dataimage = dataimage
        self.chi2opt = None
        self.isfitted = None
        self.issaved = None
        self.update_tags(dataimage.tags)

    def cvnumber(self, entry):
        if entry.startswith("cv"):
            # return the number of the curve
            return int(entry.split("_")[0][2:])
        else:
            # entry is not a curve parameter
            return None

    def freezecurve(self, k):
        # freeze the given curve parameters
        for name in self.models[k].param_names():
            self.partry[name].vary = False

    def releasecurve(self, k):
        # release the given curve parameters
        for name in self.models[k].param_names():
            self.partry[name].vary = True

    def update_from_estimate(self, estimate):
        for p, v in estimate.iteritems():
            if p in self.partry.keys():
                self.partry[p].value = v

    def scale_from_estimate(self, estimate):
        for p, change in estimate.iteritems():
            if p in self.partry.keys():
                self.partry[p].value = change[1] * self.partry[p].value + change[0]

    def set_whole_as_try(self):
        for kk in self.partry:
            if kk in self.paropt:
                self.partry[kk].value = self.paropt[kk].value

    def eval_try(self, x):
        return self.composite_model.eval(self.partry, x=x)

    def eval_opt(self, x):
        return self.composite_model.eval(self.paropt, x=x)

    def do_fit(self, data, x, weights):
        if self.composite_model is None:
            return
        try:
            result = self.composite_model.fit(
                data, params=self.partry, method=self.method, weights=weights, x=x
            )
        except Exception as e:
            print(e)
            QMessageBox.warning(self, "error", "error while trying to fit")
        self.paropt = result.params
        self.chi2opt = result.chisqr
        if result.success:
            self.isfitted = True
            self.issaved = False

    def eval_integrals_opt(self):
        # evaluate integrals of the different peaks
        return self.composite_model.eval_component_integrals(self.paropt)

    def save_opt_results(self):
        # return lines describing the results that may be saved in a result file
        # we compute the integrals
        # writing names of fitting parameters
        if self.chi2opt is None:
            return

        lig = "title "
        for kk in self.paropt:
            lig = lig + kk + " "

        integrals = self.eval_integrals_opt()

        print("computation of integrals", integrals)
        # OrderedDict of {prefix: (integral,error)}

        intnames = ""
        intvalues = ""
        interrors = ""

        for prefix in integrals.keys():
            intnames = intnames + prefix + "integral "
            value = integrals[prefix]
            intvalues = intvalues + "%g " % (value[0])
            interrors = interrors + "%g " % (value[1])

        lig = lig + intnames + "chi2 model \n"

        # writing names of fitting parameters
        title = self.dataimage.title

        lig = lig + title + " "

        for kk in self.partry:
            lig = lig + "%g " % (self.paropt[kk].value)

        lig = lig + intvalues + "%g " % (self.chi2opt)
        model = self.composite_model.name
        model = model.replace(" ", "")

        lig = lig + model + "\n"
        # print lig

        # on ecrit une ligne avec les erreurs
        lig = lig + "sigma "
        for kk in self.partry:
            if self.paropt[kk].stderr is None:
                lig = lig + "None "
            else:
                lig = lig + "%g " % (self.paropt[kk].stderr)

        lig = lig + interrors + "0. "  # no uncertainty on chi2

        lig = lig + model + "\n"
        return lig

    def update_tags(self, tags):
        # on supprime les entrees precedentes:

        for tag in self.tags:  # remove old list
            name = tag[0]
            if name in self.partry.keys():
                del self.partry[name]  # we suppress the old tag entries

        for tag in tags:  # put new list
            name = tag[0]
            self.partry.add(
                name, value=tag[1], vary=False
            )  # we reinitialize the tag entries

        self.tags = tags


class DataImage:
    # a class for describing 2D data to be fitted
    # contains x ranges, y ranges, values, errors
    def __init__(
        self,
        data,
        x_range=[0, 1],
        y_range=[0, 1],
        weights=None,
        title="",
        tags=[],
        xlabel="x",
        ylabel="y",
    ):
        self.data = data
        self.x_range = x_range
        self.y_range = y_range
        if weights is None:
            weights = np.ones_like(data)
        self.weights = weights
        self.title = title
        self.tags = tags  # list of fixed parameters (name,value) associated to the data
        self.xlabel = xlabel
        self.ylabel = ylabel
        self.rf0 = np.sum(
            self.data[np.nonzero(np.isfinite(self.data))]
            * self.weights[np.nonzero(np.isfinite(self.data))]
        )


class ImageList(QListWidget):
    """ A specialized QListWidget that displays the list
        of all images that could be fitted """

    def __init__(self, parent=None):
        QListWidget.__init__(self, parent)
        self.setSelectionMode(1)
        self.dataimages = []

    def add_item(self, dataimage):
        """ add a dataimage item to the list """
        self.dataimages.append(dataimage)
        item = QListWidgetItem(self)
        item.setText(dataimage.title)


class FunctionList(QListWidget):
    """ A specialized QListWidget that displays the list
        of functions used in the model """

    def __init__(self, parent=None):
        QListWidget.__init__(self, parent)
        self.setSelectionMode(1)
        self.current_item_number = -1
        self.initcontexts()
        self.currentRowChanged.connect(self.row_changed)
        self.itemClicked.connect(self.item_clicked)

    def add_item(self, function_name):
        """ add a function name to the list """
        item = QListWidgetItem(self)
        item.setText(function_name)
        self.current_item_number = self.count() - 1

    def row_changed(self, i):
        # activated when selected row has changed
        self.current_item_number = i

    def item_clicked(self):
        self.context.popup(QCursor.pos())

    def initcontexts(self):
        # menu pour la colonne 0
        self.context = QMenu(self)

        self.removeAction = QAction("Remove", self)
        self.context.addAction(self.removeAction)
        self.connect(self.removeAction, SIG_TRIGGERED, self.remove_item)

    def remove_item(self):
        i = self.currentRow()
        self.takeItem(i)
        self.emit(SIG_MODEL_REMOVED, i)

    def initialize(self, function_names):
        self.clear()
        for function_name in function_names:
            self.add_item(function_name)


class ParameterTable(QTableWidget):
    __implements__ = (IPanel,)
    PANEL_ID = ID_PARAMETER
    PANEL_TITLE = "parameters"
    PANEL_ICON = None  # string
    # objet qui gere les entree de parametres, l'affichage des courbes, le lancement des fits, l'enregistrement du resultat
    def __init__(self, parent=None, extendedparams=None):
        QTableWidget.__init__(self, parent=parent)

        self.setColumnCount(8)
        self.setHorizontalHeaderLabels(
            (
                "Parameters",
                "Estimation",
                "Fit result",
                "Sigma",
                "Restrains",
                "Min",
                "Max",
                "Expression",
            )
        )

        if extendedparams == None:
            self.extendedparams = ExtendedParams()
        else:
            self.extendedparams = extendedparams
        # on remet le tableau a zero : deux rangees pour un polynome du 1er degre
        self.reset()

        # on definit les differents menus contextuels
        self.initcontexts()

        # au cas ou
        self.int1 = 0

        # On connecte les signaux
        self.cellChanged.connect(self.cellChanged)
        self.cellPressed.connect(self.cellPressed)
        self.horizontalHeader().sectionPressed.connect(self.sectionClicked)

        # pas de fit sauve
        self.isfitted = False
        self.issaved = False
        self.tags = list()
        self.saveint = False
        self.ints = []
        self.updatestart = (
            False  # si coche, on active setwholeastry apres enregistrement
        )
        self.cwd = abspath

    def register_plot(self, baseplot):
        pass

    def register_panel(self, manager):
        """Register panel to plot manager"""
        pass

    def configure_panel(self):
        """Configure panel"""
        pass

    def reset(self):
        self.extendedparams.reset()
        self.initparams(settry=True)
        self.savename = None
        self.isfitted = False

    def inititems(self, listrow):
        # met pour les lignes i de listrow un item vide non editable pour colonnes 0,2,3,4
        for i in listrow:
            for j in range(8):
                self.setItem(i, j, QTableWidgetItem(0))
            self.item(i, 0).setFlags(Qt.ItemFlags(33))
            self.item(i, 2).setFlags(Qt.ItemFlags(33))
            self.item(i, 3).setFlags(Qt.ItemFlags(33))
            self.item(i, 4).setFlags(Qt.ItemFlags(33))

    def initparams(self, settry=False, setopt=False):
        blocked = self.blockSignals(True)

        # on redessine le tableau a l'aide du dictionnaire extendedparams
        # si settry, on remplit aussi les valeurs d'essai, les bornes, les expressions

        oldrc = self.rowCount()
        newrc = len(self.extendedparams.partry)
        self.setRowCount(newrc)

        if newrc > oldrc:
            # on rajoute des rangees vides
            self.inititems(range(oldrc, newrc))

        n = 0
        for entry, value in self.extendedparams.partry.iteritems():
            self.setrow(entry, n, settry, setopt)
            n += 1

        self.blockSignals(blocked)

    def setrow(self, nom, n, settry=True, setopt=False):
        blocked = self.blockSignals(True)

        ptry = self.extendedparams.partry
        popt = self.extendedparams.paropt
        self.item(n, 0).setText(nom)

        if settry:
            vtry = ptry[nom].value

            mintry = ptry[nom].min
            maxtry = ptry[nom].max
            print(nom, mintry, maxtry)
            varytry = ptry[nom].vary
            exprtry = ptry[nom].expr
            self.item(n, 1).setText("%f" % (vtry))

            if exprtry is not None:
                self.item(n, 4).setText("Expr")
                self.item(n, 7).setText(exprtry)

            elif varytry is True:
                self.item(n, 4).setText("Free")
            else:
                self.item(n, 4).setText("Fixed")

            if mintry is not None:
                self.item(n, 5).setText("%f" % (mintry))

            if maxtry is not None:
                self.item(n, 6).setText("%f" % (maxtry))

        if setopt:
            vopt = popt[nom].value
            self.item(n, 2).setText("%f" % (vopt))
            verr = popt[nom].stderr
            if verr is not None:
                self.item(n, 3).setText("%f" % (verr))

        else:
            self.item(n, 2).setText("")
            self.item(n, 3).setText("")

        self.blockSignals(blocked)  # going back to previous setting

    def initcontexts(self):
        # initialise une liste de menus contextuels pour les cases, associes a chaque colonne
        self.contexts = list(None for i in range(self.columnCount()))
        # initialise une liste de menus contextuels pour les sectionheaders, associes a chaque colonne
        self.Contexts = list(None for i in range(self.columnCount()))

        # menu pour la colonne 0

        context = QMenu(self)

        self.setastry = QAction("Set as try", self)
        context.addAction(self.setastry)
        self.connect(self.setastry, SIG_TRIGGERED, self.set_as_try)

        self.showfit = QAction("Display Fit", self)
        context.addAction(self.showfit)
        self.connect(self.showfit, SIG_TRIGGERED, self.show_fit)

        self.showtry = QAction("Display Try", self)
        context.addAction(self.showtry)
        self.connect(self.showtry, SIG_TRIGGERED, self.show_try)

        self.contexts[2] = context

        # menu pour la colonne 4
        context = QMenu(self)

        self.setfixed = QAction("Fixed", self)
        context.addAction(self.setfixed)
        self.connect(self.setfixed, SIG_TRIGGERED, self.set_fixed)

        self.setfree = QAction("Free", self)
        context.addAction(self.setfree)
        self.connect(self.setfree, SIG_TRIGGERED, self.set_free)

        self.useexpr = QAction("Expr", self)
        context.addAction(self.useexpr)
        self.connect(self.useexpr, SIG_TRIGGERED, self.use_expr)

        self.contexts[4] = context

        # menu pour le header 2
        context = QMenu(self)
        self.setwholeastry = QAction("Set whole as try", self)
        context.addAction(self.setwholeastry)
        self.connect(self.setwholeastry, SIG_TRIGGERED, self.set_whole_as_try)

        self.Contexts[2] = context

        # menu pour le header 4
        context = QMenu(self)
        self.inverserestrains = QAction("Inverse restrains", self)
        context.addAction(self.inverserestrains)
        self.connect(self.inverserestrains, SIG_TRIGGERED, self.inverse_restrains)

        self.fixall = QAction("All fixed", self)
        context.addAction(self.fixall)
        self.connect(self.fixall, SIG_TRIGGERED, self.fix_all)

        self.releaseall = QAction("All free", self)
        context.addAction(self.releaseall)
        self.connect(self.releaseall, SIG_TRIGGERED, self.release_all)

        self.Contexts[4] = context

    def cellPressed(self, int1, int2):
        self.int1 = int1

        context = self.contexts[int2]

        if context is not None:
            context.popup(QCursor.pos())

    def cellChanged(self, int1, int2):
        if int2 == 1:  # on change la valeur du parametre
            txt = str(self.item(int1, 1).text())
            entry = str(self.item(int1, 0).text())
            try:
                self.extendedparams.partry[entry].value = float(txt)
                self.emit(SIG_TRY_CHANGED)
            except ValueError:
                QMessageBox.warning("warning", "unable to convert text to float")

        if int2 == 5:  # on change la borne inferieure
            txt = str(self.item(int1, 5).text())
            entry = str(self.item(int1, 0).text())
            try:
                self.extendedparams.partry[entry].min = float(txt)
            except ValueError:
                QMessageBox.warning("warning", "unable to convert text to float")

        if int2 == 6:  # on change la borne superieure
            txt = str(self.item(int1, 6).text())
            entry = str(self.item(int1, 0).text())
            try:
                self.extendedparams.partry[entry].max = float(txt)
            except ValueError:
                QMessageBox.warning("warning", "unable to convert text to float")

        if int2 == 7:  # on change l'expression
            txt = str(self.item(int1, 7).text())
            entry = str(self.item(int1, 0).text())
            if len(txt) == 0:
                self.extendedparams.partry[entry].expr = None
                self.extendedparams.partry[entry].vary = True
            else:
                self.extendedparams.partry[entry].expr = txt
                self.item(int1, 4).setText("Expr")

    def sectionClicked(self, int2):
        # quand une colonne entiere est selectionnee valable pour les colonnes 2 et 4
        context = self.Contexts[int2]
        if context is not None:
            context.popup(QCursor.pos())

    def freeze_cv(self):
        # on met fixe tous les parametres d'une courbe
        entry = str(self.item(self.int1, 0).text())
        icurve = self.extendedparams.cv_number(entry)
        self.extendedparams.freezecurve(icurve)
        self.initparams(settry=True, setopt=True)

    def release_cv(self):
        # on met fixe tous les parametres d'une courbe
        entry = str(self.item(self.int1, 0).text())
        icurve = self.extendedparams.cv_number(entry)
        self.extendedparams.releasecurve(icurve)
        self.initparams(settry=True, setopt=True)

    def release_all(self):
        # on met fixe tous les parametres du fond
        for int1 in range(self.rowCount()):
            item = self.item(int1, 4)
            item.setText("Free")
            entry = str(self.item(int1, 0).text())
            self.extendedparams.partry[entry].vary = True

    def fix_all(self):
        # on met fixe tous les parametres du fond
        for int1 in range(self.rowCount()):
            item = self.item(int1, 4)
            item.setText("Fixed")
            entry = str(self.item(int1, 0).text())
            self.extendedparams.partry[entry].vary = False

    def set_cv(self, curvetype=None, params=None, bg=0.0):
        # icurve numero de la courbe a changer le cas echeant -1 c'est la derniere
        pass

    def rm_cv(self):
        # on supprime la courbe selectionnee
        pass

    def ch_cv(self):
        # a partir de la souris, on retrace la courbe selectionnee
        pass

    def show_fit(self):
        pass

    def show_try(self):
        pass

    def set_fixed(self):
        item = self.item(self.int1, 4)
        item.setText("Fixed")
        entry = str(self.item(self.int1, 0).text())
        self.extendedparams.partry[entry].vary = False
        self.extendedparams.partry[entry].expr = None

    def set_free(self):
        item = self.item(self.int1, 4)
        item.setText("Free")
        entry = str(self.item(self.int1, 0).text())
        self.extendedparams.partry[entry].vary = True
        self.extendedparams.partry[entry].expr = None

    def use_expr(self):
        item = self.item(self.int1, 4)
        item.setText("Expr")
        entry = str(self.item(self.int1, 0).text())

        txt = str(self.item(self.int1, 7).text())
        self.extendedparams.partry[
            entry
        ].express = txt  # in principle, should have been already set

        if len(txt) != 0:  # on met l'expression indiquee
            self.extendedparams.partry[entry].expr = txt
            self.extendedparams.partry[entry].vary = False

    def inverse_restrains(self):
        # inverse les contraintes: les parametres libres deviennent fixes et vice-versa
        for int1 in range(self.rowCount()):
            item = self.item(int1, 4)
            entry = str(self.item(int1, 0).text())
            vary = self.extendedparams.partry[entry].vary
            if vary:
                self.extendedparams.partry[entry].vary = False
                item.setText("Fixed")
            else:
                self.extendedparams.partry[entry].vary = True
                item.setText("Free")

    def set_whole_as_try(self):
        self.extendedparams.set_whole_as_try()
        self.initparams(settry=True, setopt=True)

    def set_as_try(self):
        item = self.item(self.int1, 1)
        entry = str(self.item(self.int1, 0).text())
        if entry in self.extendedparams.paropt:
            self.extendedparams.partry[entry].value = self.extendedparams.paropt[
                entry
            ].value
            item.setText("%f" % (self.extendedparams.paropt[entry].value))
            self.drawtry()


# this widget does nothing by itself but contains a layout for plots
class LayoutWidget(QWidget):
    def __init__(self, parent, orientation="horizontal"):
        self.parent = parent
        QWidget.__init__(self, parent=parent)
        if orientation == "horizontal":
            self.layout = QHBoxLayout()
        else:
            self.layout = QVBoxLayout()
        self.setLayout(self.layout)

    def addWidget(self, widget):
        self.layout.addWidget(widget)

    def save_widget(self, fname):
        """Grab widget's window and save it to filename (*.png)"""
        pixmap = QPixmap.grabWidget(self)
        pixmap.save(fname, "PNG")


class FitWidget(QWidget):
    def __init__(self, parent=None, extendedparams=None):
        QWidget.__init__(self, parent=parent)

        self.extendedparams = extendedparams

        layout = QVBoxLayout()
        self.setLayout(layout)

        self.method = QComboBox(self)
        for couple in minimizationmethods:
            self.method.addItem(couple[1])
        self.method.currentIndexChanged.connect(self.index_changed)

        self.chi2 = QLabel("chi2=")
        self.rf = QLabel("Rf=")
        self.di = QLabel("dI=")

        # signal connected in rodeo
        layout.addWidget(self.method)
        layout.addWidget(self.chi2)
        layout.addWidget(self.rf)
        layout.addWidget(self.di)

    def set_method_number(self, i):
        self.method.setCurrentIndex(i)

    def set_method_name(self):
        i = self.extendedparams.methods.index(self.extendedparams.method)
        self.method.setCurrentIndex(i)

    def index_changed(self, int):
        self.extendedparams.method = minimizationmethods[int][0]

    def set_chi2(self, chi2, rf, di):
        self.chi2.setText("chi2=%g" % chi2)
        self.rf.setText("Rf=%g" % rf)
        self.di.setText("dI=%g" % di)

    def set_difference(self, delta):
        self.difference.setText("DI=%g" % delta)

    def add_mem(self):
        # we memorize the current fit result, if any
        pass

    def remove_mem(self):
        # we suppress the corresponding entry
        pass


# this window holds the toolbar and the centralwidget
class Fit2DWindow(QMainWindow):
    def __init__(self):
        QMainWindow.__init__(self)
        self.extendedparams = ExtendedParams()
        self.configure_splitter()
        self.set_default_images()
        self.setGeometry(10, 30, 1260, 950)

        toolbar = self.addToolBar("tools")
        self.manager.add_toolbar(toolbar, id(toolbar))
        #       widget.manager.set_default_toolbar(toolbar)
        self.register_tools()
        self.create_connect()
        self.set_default_params()

        self.cwd = getcwd()
        self.savename = None  # name where to save the fit

    def set_default_images(self):
        self.data_image = make.imagenan(
            np.zeros((1, 1)), title="data image", xdata=[0, 1], ydata=[0, 1]
        )
        self.model_image = make.imagenan(
            np.zeros((1, 1)), title="model image", xdata=[0, 1], ydata=[0, 1]
        )
        self.difference_image = make.imagenan(
            np.zeros((1, 1)),
            title="difference image",
            xdata=[0, 1],
            ydata=[0, 1],
            colormap="RdBu",
        )

        z = np.empty((0,))
        self.data_hprofile = make.curve(
            z,
            z,
            title="data horizontal profile",
            marker="Diamond",
            markerfacecolor="r",
            markeredgecolor="r",
            markersize=4,
            linestyle="NoPen",
        )
        self.data_vprofile = make.curve(
            z,
            z,
            title="data vertical profile",
            marker="Diamond",
            markerfacecolor="r",
            markeredgecolor="r",
            markersize=4,
            linestyle="NoPen",
        )

        self.model_hprofile = make.curve(
            z, z, title="model horizontal profile", color="k"
        )
        self.model_vprofile = make.curve(
            z, z, title="model vertical profile", color="k"
        )

        self.difference_hprofile = make.curve(
            z, z, title="model horizontal difference", color="b"
        )
        self.difference_vprofile = make.curve(
            z, z, title="model vertical difference", color="b"
        )

        self.data_plot.add_item(self.data_image)
        self.model_plot.add_item(self.model_image)
        self.difference_plot.add_item(self.difference_image)

        self.hprofile.add_item(self.data_hprofile)
        self.hprofile.add_item(self.model_hprofile)
        self.hprofile.add_item(self.difference_hprofile)

        self.vprofile.add_item(self.data_vprofile)
        self.vprofile.add_item(self.model_vprofile)
        self.vprofile.add_item(self.difference_vprofile)

        self.data = None

    def configure_splitter(self):

        centralwidget = QSplitter(Qt.Horizontal, self)
        splitter = QSplitter(Qt.Vertical, self)
        self.setCentralWidget(centralwidget)

        centralwidget.addWidget(splitter)

        self.data_plot = ImagePlot(
            self,
            title="data",
            yreverse=False,
            lock_aspect_ratio=False,
            xunit="",
            yunit="",
        )
        self.model_plot = ImagePlot(
            self,
            title="model",
            yreverse=False,
            lock_aspect_ratio=False,
            xunit="",
            yunit="",
        )
        self.difference_plot = ImagePlot(
            self,
            title="difference",
            yreverse=False,
            lock_aspect_ratio=False,
            xunit="",
            yunit="",
        )

        self.contrast = ContrastAdjustment(self)
        self.hprofile = CurvePlot(self, title="horizontal profile")
        self.vprofile = CurvePlot(self, title="vertical profile")

        # adding layout widget with 3 plots and itemlist
        self.imagelayoutw = LayoutWidget(self, "horizontal")
        self.imagelayoutw.addWidget(self.data_plot)
        self.imagelayoutw.addWidget(self.model_plot)
        self.imagelayoutw.addWidget(self.difference_plot)
        splitter.addWidget(self.imagelayoutw)

        # adding layout widget with 3 plots with profiles
        self.plotlayoutw = LayoutWidget(self, "horizontal")
        self.plotlayoutw.addWidget(self.contrast)
        self.plotlayoutw.addWidget(self.hprofile)
        self.plotlayoutw.addWidget(self.vprofile)
        splitter.addWidget(self.plotlayoutw)

        # adding parameter table panel
        self.parametertable = ParameterTable(self, extendedparams=self.extendedparams)
        splitter.addWidget(self.parametertable)

        # adding a vertical layout with itemlist, fitwidget, functionlist
        self.vlayoutw = LayoutWidget(self, "vertical")
        self.image_list = ImageList(self)
        self.fitwidget = FitWidget(self, self.extendedparams)
        self.function_list = FunctionList(self)
        self.vlayoutw.addWidget(self.image_list)
        self.vlayoutw.addWidget(self.fitwidget)
        self.vlayoutw.addWidget(self.function_list)
        centralwidget.addWidget(self.vlayoutw)

        # registring panels to the plot manager
        self.manager = PlotManager(self)
        for plot in (
            self.data_plot,
            self.model_plot,
            self.difference_plot,
            self.hprofile,
            self.vprofile,
        ):
            self.manager.add_plot(plot)
        for panel in (self.contrast, self.parametertable):
            self.manager.add_panel(panel)

    def register_tools(self):
        self.manager.register_all_image_tools()
        self.addmodeltool = self.manager.add_tool(AddModelTool, list_of_2D_models)
        self.spottool = self.manager.add_tool(
            CircularActionToolCXY, self.estimate_spot, self.scale_spot
        )
        self.leveltool = self.manager.add_tool(MultiPointTool, self.estimate_bg)
        self.runtool = self.manager.add_tool(RunTool)
        self.savefittool = self.manager.add_tool(SaveFitTool)
        self.preftool = self.manager.add_tool(PrefTool)
        self.showweightstool = self.manager.add_tool(ShowWeightsTool)

    def estimate_bg(self, plot, pts):
        if plot == self.data_plot:
            intensities = []
            for pt in pts:
                intensities.append(self.data_image.get_data(pt[0], pt[1]))
            i = self.function_list.current_item_number  # number of the curve to ajust
            prefix = "cv%d_" % i
            estimated_values = {prefix + "bg": np.mean(np.array(intensities))}
            if len(pts) > 2:
                # try:
                mat = np.ones((3, 3))
                mat[:, 0:2] = pts
                sol = np.linalg.solve(mat, intensities)
                estimated_values = {
                    prefix + "slope_x": sol[0],
                    prefix + "slope_y": sol[1],
                    prefix + "bg": sol[2],
                }
            # except np.linalg.LinAlgError:
            #   pass

            self.extendedparams.update_from_estimate(estimated_values)
            self.parametertable.initparams(settry=True)
            self.update_try()

    def estimate_spot(self, plot, p0, p1):
        # from graphical positions of the circle drawn, estimate the shape of the spot
        loc0, loc1, width0, width1 = get_xywxwy_cxy(plot, p0, p1)
        i = self.function_list.current_item_number  # number of the curve to ajust
        prefix = "cv%d_" % i
        estimated_values = {
            prefix + "loc0": loc0,
            prefix + "loc1": loc1,
            prefix + "width0": width0,
            prefix + "width1": width1,
        }
        self.extendedparams.update_from_estimate(estimated_values)
        self.parametertable.initparams(settry=True)
        self.update_try()

    def scale_spot(self, plot, key):
        i = self.function_list.current_item_number  # number of the curve to ajust
        prefix = "cv%d_" % i

        zaxis = plot.colormap_axis
        # _min,_max=
        _min, _max = plot.get_axis_limits(zaxis)

        shift = 0.0
        scale = 1.0

        if key == 42:
            scale = 1.1
        elif key == 47:
            scale = 1.0 / 1.1

        if key == 43:
            shift = (_max - _min) / 10.0
        elif key == 45:
            shift = -(_max - _min) / 10.0

        estimated_values = {
            prefix + "amp": (shift, scale),
            prefix + "bg": (shift, scale),
        }
        self.extendedparams.scale_from_estimate(estimated_values)
        self.parametertable.initparams(settry=True)
        self.update_try()

    def create_connect(self):
        self.connect(self.parametertable, SIG_TRY_CHANGED, self.update_try)
        self.connect(self.addmodeltool, SIG_MODEL_ADDED, self.add_model_class)
        self.runtool.SIG_VALIDATE_TOOL.connect(self.do_fit)
        self.savefittool.SIG_VALIDATE_TOOL.connect(self.save_fit)
        self.image_list.currentRowChanged.connect(self.image_list_row_changed)
        self.connect(self.preftool.showtagsaction, SIG_TRIGGERED, self.show_tags)
        self.connect(
            self.preftool.saveprefaction, SIG_TRIGGERED, self.set_save_filename
        )
        self.connect(self.function_list, SIG_MODEL_REMOVED, self.remove_model)
        self.showweightstool.SIG_VALIDATE_TOOL.connect(self.show_weigths)
        self.connect(self.preftool.showdiffaction, SIG_TRIGGERED, self.show_diff)

    def set_default_params(self):
        # create a fit with a linear bg and a lorenzian polar function
        self.add_model_class(0)
        self.add_model_class(1)

    def add_model_class(self, i):
        modelclass = self.addmodeltool.list_of_models[i]
        added_model = self.extendedparams.add_model_class(modelclass)
        self.function_list.add_item(added_model.name)
        self.parametertable.initparams(settry=True)
        if self.data is not None:
            self.update_try()

    def remove_model(self, i):
        self.extendedparams.remove_model_class(i)
        self.parametertable.initparams(
            settry=True
        )  # refill the parametertable with new values
        self.function_list.initialize(
            list((model.name for model in self.extendedparams.models))
        )  # refill the function table

    def set_save_filename(self):
        self.savename = str(
            QFileDialog.getSaveFileName(
                None, "Save fit parameters", self.cwd, filter="*.txt"
            )
        )
        if len(self.savename) == 0:
            self.savename = None
        else:
            self.cwd = path.dirname(path.realpath(self.savename))

    def save_fit(self):
        # sauvegarde des donnees
        if not self.extendedparams.isfitted:
            # no fit has been performed!
            return

        if self.savename is None:
            self.set_save_filename()  # definit le nom de fichier
            if self.savename is None:
                return

        fic = open(self.savename, "a")
        fic.write(self.extendedparams.save_opt_results())
        fic.close()

        # on sauve la figure de fit
        ific = 1

        figshortname = self.dataimage.title + "_%.3d.png" % (ific)
        figname = path.join(self.cwd, figshortname)
        while path.isfile(figname):
            ific = ific + 1
            figshortname = self.dataimage.title + "_%.3d.png" % (ific)
            figname = path.join(self.cwd, figshortname)
        print(figname, " sauve")
        self.imagelayoutw.save_widget(figname)
        if self.preftool.restartaction.isChecked():
            self.extendedparams.set_whole_as_try()
            self.parametertable.initparams(settry=True)
        self.extendedparams.issaved = True

    def show_tags(self):
        pass

    def add_data(
        self,
        data,
        x_range=[0.0, 1.0],
        y_range=[0.0, 1.0],
        weights=None,
        title="",
        tags=[],
        xlabel="x",
        ylabel="y",
    ):
        dataimage = DataImage(
            data, x_range, y_range, weights, title, tags, xlabel, ylabel
        )
        self.image_list.add_item(dataimage)
        self.dataimage = dataimage  # pointer to the current dataimage
        self.set_image(dataimage)

    def image_list_row_changed(self, i):
        # the current image list row has changed
        self.dataimage = self.image_list.dataimages[i]
        if self.extendedparams.isfitted and not self.extendedparams.issaved:
            # a fit has been performed but not saved
            i = QMessageBox.question(
                self, "save", "do you want to save the fit?", "Yes", "No", "Cancel"
            )

            if i == 0:
                self.save_fit()

        self.set_image(self.dataimage)

    def set_image(self, dataimage):
        # set image to fit where dataimage contains data: a 2D array, xdata and ydata: the limits, weights: 1/sigma
        # update image representation
        self.data_image.set_xdata(*dataimage.x_range)
        self.data_image.set_ydata(*dataimage.y_range)
        self.data_image.set_data(
            np.array(dataimage.data)
        )  # ImageItem.set_data does not make a copy

        self.data_plot.set_title(dataimage.title)

        self.model_image.set_xdata(*dataimage.x_range)
        self.model_image.set_ydata(*dataimage.y_range)
        self.model_image.set_data(np.array(dataimage.data))

        self.difference_image.set_xdata(*dataimage.x_range)
        self.difference_image.set_ydata(*dataimage.y_range)
        self.difference_image.set_data(np.array(dataimage.data))

        # update axis labels
        self.data_plot.set_axis_title("left", dataimage.ylabel)
        self.data_plot.set_axis_title("bottom", dataimage.xlabel)
        self.model_plot.set_axis_title("left", dataimage.ylabel)
        self.model_plot.set_axis_title("bottom", dataimage.xlabel)
        self.difference_plot.set_axis_title("left", dataimage.ylabel)
        self.difference_plot.set_axis_title("bottom", dataimage.xlabel)

        # update values to be fitted
        self.fit_indices = np.nonzero(np.isfinite(dataimage.data))  # indices to fit
        self.model_image.data[self.fit_indices] = 0.0  # start from 0

        self.data = dataimage.data[self.fit_indices]  # value at indices, 1D array!

        if dataimage.weights is not None:
            self.weights = dataimage.weights[
                self.fit_indices
            ]  # error at indices, 1D array!
        else:
            self.weights = None

        sx = dataimage.data.shape[1]
        sy = dataimage.data.shape[0]
        fx = (dataimage.x_range[1] - dataimage.x_range[0]) / sx
        fy = (dataimage.y_range[1] - dataimage.y_range[0]) / sy

        # we put first x and y in the list
        self.x = [
            self.fit_indices[1] * fx + dataimage.x_range[0] + fx / 2.0,
            self.fit_indices[0] * fy + dataimage.y_range[0] + fy / 2.0,
        ]

        hrange = np.arange(dataimage.x_range[0] + fx / 2, dataimage.x_range[1], fx)
        vrange = np.arange(dataimage.y_range[0] + fy / 2, dataimage.y_range[1], fy)

        array1 = np.nan_to_num(dataimage.data)
        array2 = np.isfinite(dataimage.data)

        sum1 = np.sum(array1, axis=0) / np.sum(array2, axis=0)
        sum2 = np.sum(array1, axis=1) / np.sum(array2, axis=1)

        hrange = hrange[np.isfinite(sum1)]
        vrange = vrange[np.isfinite(sum2)]

        self.data_hprofile.set_data(hrange, sum1[np.isfinite(sum1)])
        self.data_vprofile.set_data(vrange, sum2[np.isfinite(sum2)])

        if self.preftool.scaleprefaction.isChecked():
            self.data_plot.do_autoscale(replot=True)
        else:
            self.data_plot.replot()
        self.data_plot.update_colormap_axis(self.data_image)

        self.extendedparams.set_dataimage(dataimage)
        self.parametertable.initparams(
            settry=True
        )  # if tags have changed, put them into the table
        self.update_try()

    def show_weigths(self):
        if self.showweightstool.action.isChecked():
            self.data_image.set_data(self.dataimage.weights)
        else:
            self.data_image.set_data(self.dataimage.data)
        self.data_plot.replot()

    def show_diff(self):
        self.difference_hprofile.setVisible(self.preftool.showdiffaction.isChecked())
        self.difference_vprofile.setVisible(self.preftool.showdiffaction.isChecked())
        self.hprofile.replot()
        self.vprofile.replot()

    def update_try(self):
        # evaluate model with try parameters and update model and difference image
        values = self.extendedparams.eval_try(self.x)
        self.model_image.data[self.fit_indices] = values

        diff = values - self.data
        self.difference_image.data[self.fit_indices] = diff
        diff = diff * self.weights
        chi2 = np.sum((diff) ** 2)

        di = np.sum((diff))
        rf = np.sum(np.absolute(diff)) / self.dataimage.rf0

        self.fitwidget.set_chi2(chi2, rf, di)
        self.update_images_and_profiles()

    def update_opt(self):
        # evaluate model with try parameters and update model and difference image
        values = self.extendedparams.eval_opt(self.x)
        self.model_image.data[self.fit_indices] = values

        diff = values - self.data
        self.difference_image.data[self.fit_indices] = diff
        diff = diff * self.weights
        chi2 = np.sum((diff) ** 2)

        di = np.sum((diff))
        rf = np.sum(np.absolute(diff)) / self.dataimage.rf0

        self.fitwidget.set_chi2(chi2, rf, di)
        self.update_images_and_profiles()

    def update_images_and_profiles(self):

        hrange = self.data_hprofile.get_data()[0]
        vrange = self.data_vprofile.get_data()[0]

        array1 = np.nan_to_num(self.model_image.data)
        array2 = np.isfinite(self.model_image.data)

        sum1 = np.sum(array1, axis=0) / np.sum(array2, axis=0)
        sum2 = np.sum(array1, axis=1) / np.sum(array2, axis=1)

        self.model_hprofile.set_data(hrange, sum1[np.isfinite(sum1)])
        self.model_vprofile.set_data(vrange, sum2[np.isfinite(sum2)])

        array1 = np.nan_to_num(self.difference_image.data)
        array2 = np.isfinite(self.difference_image.data)

        sum1 = np.sum(array1, axis=0) / np.sum(array2, axis=0)
        sum2 = np.sum(array1, axis=1) / np.sum(array2, axis=1)

        self.difference_hprofile.set_data(hrange, sum1[np.isfinite(sum1)])
        self.difference_vprofile.set_data(vrange, sum2[np.isfinite(sum2)])

        if self.preftool.samescaleaction.isChecked():
            _min, _max = _nanmin(self.dataimage.data), _nanmax(self.dataimage.data)
            self.model_image.set_lut_range([_min, _max])
        elif self.preftool.scaleprefaction.isChecked():
            self.model_image.auto_lut_scale()

        if self.preftool.scaleprefaction.isChecked():
            self.difference_image.auto_lut_scale_sym()
            self.model_plot.do_autoscale(replot=True)
            self.difference_plot.do_autoscale(replot=True)
            self.hprofile.do_autoscale(replot=True)
            self.vprofile.do_autoscale(replot=True)
        else:
            self.model_plot.replot()
            self.difference_plot.replot()
            self.hprofile.replot()
            self.vprofile.replot()
        self.model_plot.update_colormap_axis(self.model_image)
        self.difference_plot.update_colormap_axis(self.difference_image)

    def do_fit(self):
        self.extendedparams.do_fit(self.data, self.x, self.weights)
        self.update_opt()
        self.parametertable.initparams(setopt=True)


def test(win):
    # make a test model
    xmin = -1.0
    xmax = 1.0  # bornes
    ymin = -1.0
    ymax = 1.0

    data = np.random.rand(100, 100)
    x = np.arange(xmin, xmax, 0.02)
    y = np.arange(ymin, ymax, 0.02)
    xv, yv = np.meshgrid(x, y)
    data = data + 10.0 / (1.0 + xv * xv + yv * yv)
    tags = [("Qz", 0.15)]
    win.add_data(data, (xmin, xmax), (ymin, ymax), title="test1", tags=tags)

    xmin = -1.2
    xmax = 1.2  # bornes
    ymin = -1.5
    ymax = 1.5
    data = np.random.rand(150, 120)
    x = np.arange(xmin, xmax, 0.02)
    y = np.arange(ymin, ymax, 0.02)
    xv, yv = np.meshgrid(x, y)
    data = data + 5.0 / (1.0 + xv * xv + 0.5 * yv * yv)
    tags = [("l", 0.3)]
    win.add_data(data, (xmin, xmax), (ymin, ymax), title="test2", tags=tags)


if __name__ == "__main__":

    from guidata import qapplication

    _app = qapplication()
    # win = FitDialog()
    win = Fit2DWindow()
    test(win)

    """

    x=np.nonzero(data)
    fx=(xmax-xmin)/s
    fy=(ymax-ymin)/s

    xx=(np.array(x[0])+0.5)*fx+xmin
    xy=(np.array(x[1])+0.5)*fy+ymin

    models=(Linear2DModel,Lorenzian2DModel)

    win.extendedparams.add_model_class(Linear2DModel)
    win.extendedparams.add_model_class(Lorenzian2DModel)
    win.parametertable.initparams(settry=True)


    data[x]=win.extendedparams.composite_model.eval(win.extendedparams.partry,x=[xx,xy])
    image1=make.image(data,title='test',xdata=[xmin,xmax],ydata=[ymin,ymax])
    """
    win.show()
    # test=ParameterTable(extendedparams=ExtendedParams())
    # test.show()
    _app.exec_()



from typing import NamedTuple, Optional, Text, Union
from os.path import join

from functools import partial
from h5py import Dataset, File, Group

from ..util import as_string

# Generic hdf5 access types.

class DatasetPathContains(NamedTuple):
    path: Text


class DatasetPathWithAttribute(NamedTuple):
    attribute: Text
    value: bytes


class HItem(NamedTuple):
    name: Text
    optional: bool


DatasetPath = Union[DatasetPathContains, DatasetPathWithAttribute, HItem]

class DatasetPathOr(NamedTuple):
    path1: DatasetPath
    path2: DatasetPath


def _v_attrs(attribute: Text, value: Text, _name: Text, obj) -> Dataset:
    """visite each node and check the attribute value"""
    if isinstance(obj, Dataset):
        if attribute in obj.attrs and obj.attrs[attribute] == value:
            return obj


def _v_item(key: Text, name: Text, obj: Dataset) -> Dataset:
    """visite each node and check that the path contain the key"""
    if key in name:
        return obj


def get_dataset(h5file: File, path: DatasetPath) -> Optional[Dataset]:
    res = None
    if isinstance(path, DatasetPathContains):
        res = h5file.visititems(partial(_v_item, path.path))
    elif isinstance(path, DatasetPathWithAttribute):
        res = h5file.visititems(partial(_v_attrs, path.attribute, path.value))
    elif isinstance(path, HItem):
        res = h5file.visititems(partial(_v_item, join("scan_data", path.name)))
        if not path.optional and res is None:
            raise Exception("Can not find : {}".format(path))
    elif isinstance(path, DatasetPathOr):
        res = get_dataset(h5file, path.path1)
        if res is None:
            res = get_dataset(h5file, path.path2)
    return res


# tables here...


class GroupPathWithAttribute(NamedTuple):
    attribute: Text
    value: bytes


class GroupPathNxClass(NamedTuple):
    value: bytes


GroupPath = Union[GroupPathWithAttribute, GroupPathNxClass, str]


def _g_attrs(attribute: Text, value: Text, _name: Text, obj) -> Optional[Group]:
    """visite each node and check the attribute value"""
    if isinstance(obj, Group):
        if attribute in obj.attrs and obj.attrs[attribute] == value:
            return obj
    return None


def get_nxclass(h5file: File, gpath: GroupPath) -> Optional[Group]:
    res = None
    if isinstance(gpath, GroupPathWithAttribute):
        res = h5file.visititems(partial(_g_attrs, gpath.attribute, gpath.value))
    elif isinstance(gpath, GroupPathNxClass):
        res = h5file.visititems(partial(_g_attrs, "NX_class", gpath.value))
    elif isinstance(gpath, str):
        res = h5file.visititems(partial(_g_attrs, "NX_class", gpath.encode()))

    return res


def node_as_string(node):
    if node.shape == ():
        content = node
    else:
        content = node[0]
    return as_string(content)



from __future__ import print_function, with_statement, division

import os


def run(args):
    """Parameters
        args: string
            String as if typed in terminal. The string must consist
            of the location of the configuration file and the command
            for specifying the jobs that need to be processed.
            All additonal configuration file overides can be included

        Returns
        A tuple of binoculars spaces

        Examples:
        >>> space = binoculars.run('config.txt 10')
        >>> space[0]
        Axes (3 dimensions, 2848 points, 33.0 kB) {
            Axis qx (min=-0.01, max=0.0, res=0.01, count=2)
            Axis qy (min=-0.04, max=-0.01, res=0.01, count=4)
            Axis qz (min=0.48, max=4.03, res=0.01, count=356)
        }
    """

    import binoculars.main

    binoculars.util.register_python_executable(__file__)
    main = binoculars.main.Main.from_args(args.split(" "))

    if isinstance(main.result, binoculars.space.Multiverse):
        return main.result.spaces
    if type(main.result) == bool:
        filenames = main.dispatcher.config.destination.final_filenames()
        return tuple(binoculars.space.Space.fromfile(fn) for fn in filenames)


def load(filename, key=None):
    """ Parameters
        filename: string
            Only hdf5 files are acceptable
        key: a tuple with slices in as much dimensions as the space is

        Returns
        A binoculars space

        Examples:
        >>> space = binoculars.load('test.hdf5')
        >>> space
        Axes (3 dimensions, 2848 points, 33.0 kB) {
            Axis qx (min=-0.01, max=0.0, res=0.01, count=2)
            Axis qy (min=-0.04, max=-0.01, res=0.01, count=4)
            Axis qz (min=0.48, max=4.03, res=0.01, count=356)
        }
    """
    import binoculars.space

    if os.path.exists(filename):
        return binoculars.space.Space.fromfile(filename, key=key)
    else:
        raise IOError("File '{0}' does not exist".format(filename))


def save(filename, space):
    """
        Save a space to file

        Parameters
        filename: string
            filename to which the data is saved. '.txt', '.hdf5' are supported.
        space: binoculars space
            the space containing the data that needs to be saved

        Examples:
        >>> space
        Axes (3 dimensions, 2848 points, 33.0 kB) {
            Axis qx (min=-0.01, max=0.0, res=0.01, count=2)
            Axis qy (min=-0.04, max=-0.01, res=0.01, count=4)
            Axis qz (min=0.48, max=4.03, res=0.01, count=356)
        }
        >>> binoculars.save('test.hdf5', space)
    """

    import binoculars.space
    import binoculars.util

    if isinstance(space, binoculars.space.Space):
        ext = os.path.splitext(filename)[-1]
        if ext == ".txt":
            binoculars.util.space_to_txt(space, filename)
        elif ext == ".edf":
            binoculars.util.space_to_edf(space, filename)
        else:
            space.tofile(filename)
    else:
        raise TypeError("'{0!r}' is not a binoculars space".format(space))


def plotspace(
    space,
    log=True,
    clipping=0.0,
    fit=None,
    norm=None,
    colorbar=True,
    labels=True,
    **plotopts
):
    """plots a space with the correct axes. The space can be either one
        or two dimensinal.

        Parameters
        space: binoculars space
            the space containing the data that needs to be plotted
        log: bool
            axes or colorscale logarithmic
        clipping: 0 < float < 1
            cuts a lowest and highst value on the color scale
        fit: numpy.ndarray

            same shape and the space. If one dimensional the fit will
            be overlayed.
        norm: matplotlib.colors
            object defining the colorscale
        colorbar: bool
            show or not show the colorbar
        labels: bool
            show or not show the labels
        plotopts: keyword arguments
            keywords that will be accepted by matplotlib.pyplot.plot
            or matplotlib.pyplot.imshow

        Examples:
        >>> space
        Axes (3 dimensions, 2848 points, 33.0 kB) {
            Axis qx (min=-0.01, max=0.0, res=0.01, count=2)
            Axis qy (min=-0.04, max=-0.01, res=0.01, count=4)
            Axis qz (min=0.48, max=4.03, res=0.01, count=356)
        }
        >>> binoculars.plotspace('test.hdf5')
    """

    import matplotlib.pyplot as pyplot
    import binoculars.plot
    import binoculars.space

    if isinstance(space, binoculars.space.Space):
        if space.dimension == 3:
            from mpl_toolkits.mplot3d import Axes3D  # noqa

            ax = pyplot.gcf().gca(projection="3d")
            return binoculars.plot.plot(
                space,
                pyplot.gcf(),
                ax,
                log=log,
                clipping=clipping,
                fit=None,
                norm=norm,
                colorbar=colorbar,
                labels=labels,
                **plotopts
            )
        if fit is not None and space.dimension == 2:
            ax = pyplot.gcf().add_subplot(121)
            binoculars.plot.plot(
                space,
                pyplot.gcf(),
                ax,
                log=log,
                clipping=clipping,
                fit=None,
                norm=norm,
                colorbar=colorbar,
                labels=labels,
                **plotopts
            )
            ax = pyplot.gcf().add_subplot(122)
            return binoculars.plot.plot(
                space,
                pyplot.gcf(),
                ax,
                log=log,
                clipping=clipping,
                fit=fit,
                norm=norm,
                colorbar=colorbar,
                labels=labels,
                **plotopts
            )
        else:
            return binoculars.plot.plot(
                space,
                pyplot.gcf(),
                pyplot.gca(),
                log=log,
                clipping=clipping,
                fit=fit,
                norm=norm,
                colorbar=colorbar,
                labels=labels,
                **plotopts
            )
    else:
        raise TypeError("'{0!r}' is not a binoculars space".format(space))


def transform(space, labels, resolutions, exprs):
    """transformation of the coordinates.

        Parameters
        space: binoculars space
        labels: list
            a list of length N with the labels
        resolutions: list
            a list of length N with the resolution per label
        exprs: list
            a list of length N with strings containing the expressions
            that will be evaluated.  all numpy funtions can be called
            without adding 'numpy.' to the functions.
        Returns
        A binoculars space of dimension N with labels and resolutions
        specified in the input

        Examples:
        >>> space = binoculars.load('test.hdf5')
        >>> space
        Axes (3 dimensions, 2848 points, 33.0 kB) {
            Axis qx (min=-0.01, max=0.0, res=0.01, count=2)
            Axis qy (min=-0.04, max=-0.01, res=0.01, count=4)
            Axis qz (min=0.48, max=4.03, res=0.01, count=356)
        }
        >>> newspace = binoculars.transform(space, ['twotheta'], [0.003], ['2 * arcsin(0.51 * (sqrt(qx**2 + qy**2 + qz**2) / (4 * pi)) / (pi * 180))'])  # noqa
        >>> newspace
        Axes (1 dimensions, 152 points, 1.0 kB) {
            Axis twotheta (min=0.066, max=0.519, res=0.003, count=152)
        }

    """
    import binoculars.util
    import binoculars.space

    if isinstance(space, binoculars.space.Space):
        transformation = binoculars.util.transformation_from_expressions(space, exprs)
        newspace = space.transform_coordinates(resolutions, labels, transformation)
    else:
        raise TypeError("'{0!r}' is not a binoculars space".format(space))
    return newspace


def fitspace(space, function, guess=None):
    """
        fit the space data.

        Parameters
        space: binoculars space
        function: list
            a string with the name of the desired function. supported are:
            lorentzian (automatically selects 1d or 2d), gaussian1d and voigt1d
        guess: list
            a list of length N with the resolution per label

        Returns
        A binoculars fit object.

        Examples:
        >>> fit = binoculars.fitspace(space, 'lorentzian')
        >>> print(fit.summary)
            I: 1.081e-07 +/- inf
            loc: 0.3703 +/- inf
            gamma: 0.02383 +/- inf
            slope: 0.004559 +/- inf
            offset: -0.001888 +/- inf
        >>> parameters = fit.parameters
        >>> data = fit.fitdata
        >>> binoculars.plotspace(space, fit = data)
    """

    import binoculars.fit

    if isinstance(space, binoculars.space.Space):
        fitclass = binoculars.fit.get_class_by_name(function)
        return fitclass(space, guess)
    else:
        raise TypeError("'{0!r}' is not a binoculars space".format(space))


def info(filename):
    """
        Explore the file without loading the file, or after loading the file

        Parameters
        filename: filename or space

        Examples:
        >>> print binoculars.info('test.hdf5')
        Axes (3 dimensions, 46466628 points, 531.0 MB) {
            Axis H (min=-0.1184, max=0.0632, res=0.0008, count=228)
            Axis K (min=-1.1184, max=-0.9136, res=0.0008, count=257)
            Axis L (min=0.125, max=4.085, res=0.005, count=793)
        }
        ConfigFile{
           [dispatcher]
           [projection]
           [input]
        }
        origin = test.hdf5
        >>> space = binoculars.load('test.hdf5')
        >>> print binoculars.info(space)
        Axes (3 dimensions, 46466628 points, 531.0 MB) {
            Axis H (min=-0.1184, max=0.0632, res=0.0008, count=228)
            Axis K (min=-1.1184, max=-0.9136, res=0.0008, count=257)
            Axis L (min=0.125, max=4.085, res=0.005, count=793)
        }
        ConfigFile{
           [dispatcher]
           [projection]
           [input]
        }
        origin = test.hdf5

    """

    import binoculars.space

    ret = ""
    if isinstance(filename, binoculars.space.Space):
        ret += "{!r}\n{!r}".format(filename, filename.config)
    elif type(filename) == str:
        if os.path.exists(filename):
            try:
                axes = binoculars.space.Axes.fromfile(filename)
            except Exception as e:
                raise IOError(
                    "{0}: unable to load Space: {1!r}".format(filename, e)
                )  # noqa
            ret += "{!r}\n".format(axes)
            try:
                config = binoculars.util.ConfigFile.fromfile(filename)
            except Exception as e:
                raise IOError(
                    "{0}: unable to load util.ConfigFile: {1!r}".format(filename, e)
                )  # noqa
            ret += "{!r}".format(config)
        else:
            raise IOError("File '{0}' does not exist".format(filename))
    return ret



# -*- coding: utf-8 -*-
"""
Created on Tue Oct 08 10:39:12 2013

@author: Prevot
"""


import numpy as np
from scipy.interpolate import griddata

from guidata.qt import QtCore, QtGui

import os
import tables


# from guidata.qt.QtGui import QFont
# from guidata.qt.QtCore import Qt
from guiqwt.plot import CurveDialog
from guiqwt.builder import make
from guiqwt.signals import (
    SIG_ACTIVE_ITEM_CHANGED,
    SIG_START_TRACKING,
    SIG_MOVE,
    SIG_STOP_NOT_MOVING,
    SIG_STOP_MOVING,
)
from guiqwt.events import QtDragHandler, setup_standard_tool_filter
from guiqwt.tools import InteractiveTool, DefaultToolbarID, CommandTool
from guiqwt.interfaces import ICurveItemType
from guiqwt.config import _
from guiqwt.shapes import Marker
from guiqwt.interfaces import ITrackableItemType

# from guidata.qthelpers import get_std_icon
from guidata.configtools import get_icon
from reciprocal2D import D2Dview

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

# import guidata
# _app = guidata.qapplication()
degtorad = np.pi / 180.0

"""
paramsdict = {a[0]:a[1] for a in params}
paramslist = list(a[0] for a in params)
"""


class MeshParameters:
    # classe pour les parametres de construction d'une map 3D
    def __init__(
        self,
        Nx=1,
        Ny=1,
        xmin=0.0,
        ymin=0.0,
        xmax=0.0,
        ymax=0.0,
        ix=0,
        iy=0,
        iz=0,
        ilog=0,
        io=0,
        angle=0.0,
    ):
        self.Nx = Nx
        self.Ny = Ny
        self.xmin = xmin
        self.ymin = ymin
        self.xmax = xmax
        self.ymax = ymax
        self.ix = ix
        self.iy = iy
        self.iz = iz
        self.ilog = ilog  # intensite en echelle log
        self.io = io  # utiliser la matrice d'orientation
        self.angle = angle  # rajouter cet angle
        self.bc = False  # correction de fond sur chaque scan
        self.b1 = 0.0  # percentile min pour estimation du background
        self.b2 = 10.0  # percentile max pour estimation du background


class DataFilter:
    # classe pour les parametres de filtre
    def __init__(self, value=0.0, role="none"):
        self.ifil = -1
        self.value = value
        self.role = role
        self.roles = list(("none", "high-pass", "low-pass", "cut"))
        self.irole = self.roles.index(self.role)


class PrefParameters:
    # classe pour les parametres de preference
    # -> affichage du nom des moteurs
    # -> mise a jour automatique du fit
    def __init__(self):
        self.namedisplays = list(
            (
                "my suggestion",
                "short name",
                "short and family name",
                "full name",
                "nxs name",
            )
        )
        self.inamedisplay = 0
        self.autoupdate = True
        self.followscans = True


class DataSet:
    # definit une classe pour rentrer les donnees associees a un fichier nxs
    def __init__(self, longname, shortname, datafilter=None, pref=None):
        if pref is None:
            pref = PrefParameters()
        self.shortname = shortname
        self.longname = longname

        try:
            fichier = tables.openFile(longname)
            self.nodedatasizes = list()  # liste des longueurs des  tableaux de donnees
            # print shortname,fichier.listNodes('/')[0].start_time[0],fichier.listNodes('/')[0].end_time[0]
            # for leaf in fichier.__iter__():
            #
            #    if leaf._v_name=="temperature":
            #        print leaf._v_name,leaf.read()[0]

            for leaf in fichier.listNodes("/")[0].scan_data:
                self.nodedatasizes.append(leaf.shape[0])

            self.npts = max(self.nodedatasizes)

            # on ne selectionne que les noeuds qui ont la meme longueur, il peut y avoir des donnes avec des tableaux de taille plus petite par exemple 1, on laisse tomber
            self.nodenames = list()  # nom des noeuds (ex data_01)
            self.nodelongnames = list()  # nom comprehensible des noeuds (complet)
            self.nodenicknames = list()  # diminutif pour affichage
            self.data = np.empty(0)  # creation d'un tableau vide de depart

            # print pref.inamedisplay
            for leaf in fichier.listNodes("/")[0].scan_data:
                if leaf.shape[0] == self.npts:
                    self.nodenames.append(leaf.name)
                    try:
                        nodelongname = leaf.attrs.long_name
                    except:
                        nodelongname = ""
                    # nodelongname=leaf.attrs.long_name[leaf.attrs.long_name.rfind('/')+1:]
                    if len(nodelongname) == 0:
                        nodelongname = (
                            leaf.name
                        )  # si pas de nom long on garde le nom nxs
                    self.nodelongnames.append(nodelongname)
                    self.data = np.concatenate(
                        (self.data, leaf.read()[1:])
                    )  # on ajoute les donnees au tableau np, on enleve le premier point

                    if pref.inamedisplay <= 1:
                        nodenickname = nodelongname.split("/")[
                            -1
                        ]  # on prend le dernier
                        self.nodenicknames.append(nodenickname)

                    elif pref.inamedisplay == 2:
                        try:
                            namesplit = nodelongname.split("/")
                            nodenickname = (
                                namesplit[-2] + "/" + namesplit[-1]
                            )  # on prend les deux derniers si possible
                            self.nodenicknames.append(nodenickname)
                        except:
                            self.nodenicknames.append(nodelongname)

                    elif pref.inamedisplay == 3:
                        self.nodenicknames.append(
                            nodelongname
                        )  # on prend le nom long complet

                    elif pref.inamedisplay == 4:
                        self.nodenicknames.append(leaf.name)  # on prend le nom nxs

        except ValueError:
            print("probleme le fichier ", longname, "est corrompu")
            self.npts = 0
            self.nmotors = 0
            self.mins = np.empty(0)
            self.maxs = np.empty(0)
            self.data = np.empty(0)
            fichier.close()
            return
        except tables.exceptions.NoSuchNodeError:
            print("probleme le fichier ", longname, "est corrompu")
            self.npts = 0
            self.nmotors = 0
            self.mins = np.empty(0)
            self.maxs = np.empty(0)
            self.data = np.empty(0)
            fichier.close()
            return
        else:
            fichier.close()

        self.npts = self.npts - 1  # on a enleve le premier point
        self.nmotors = len(self.nodenames)  # nombre de colonnes retenues

        # si les preferences de display sont "my suggestion" on va regarder si un nom figure plusieurs fois
        # dans ce cas, on choisit de prendre le nom plus long

        if pref.inamedisplay == 0:
            for i in range(self.nmotors - 1):  # pas la peine de faire le dernier point!

                nickname = self.nodenicknames[i]

                if nickname in self.nodenicknames[i + 1 :]:  # alors item en double
                    nodelongname = self.nodelongnames[i]
                    namesplit = nodelongname.split("/")
                    try:
                        nodenickname = (
                            namesplit[-2] + "/" + namesplit[-1]
                        )  # on prend les deux derniers
                    except:
                        nodenickname = nodelongname  # on prend les deux derniers

                    self.nodenicknames[i] = nodenickname

                    j = i
                    try:
                        while 1:
                            j = self.nodenicknames.index(j + 1)
                            self.nodenicknames[
                                j
                            ] = nodenickname  # attention, on ne garantit pas que nodenickname!=nickname

                    except ValueError:
                        pass

        self.data = self.data.reshape((self.nmotors, self.npts))
        # print self.data[0]
        test = np.any(
            self.data != 0, axis=0
        )  # si une valeur non nulle, la condition est verifiee
        # print test
        self.data = np.compress(test, self.data, axis=1)

        if datafilter is not None:
            # on filtre les valeurs en regardant la condition sur le filtre
            # print datafilter.role,datafilter.ifil
            if datafilter.role != "none" and datafilter.ifil > -1:
                # print "irole=",datafilter.irole
                if datafilter.irole == 1:
                    # print "on laisse si col",datafilter.ifil,">",datafilter.value
                    # print self.data[datafilter.ifil]>datafilter.value
                    self.data = np.compress(
                        self.data[datafilter.ifil] > datafilter.value, self.data, axis=1
                    )
                elif datafilter.irole == 2:
                    # print "on laisse si col",datafilter.ifil,"<",datafilter.value
                    self.data = np.compress(
                        self.data[datafilter.ifil] < datafilter.value, self.data, axis=1
                    )
                elif datafilter.irole == 3:
                    # print "on laisse si col",datafilter.ifil,"!=",datafilter.value
                    self.data = np.compress(
                        self.data[datafilter.ifil] != datafilter.value,
                        self.data,
                        axis=1,
                    )

        self.npts = self.data.shape[1]  # nombre de points non totalement nuls

        if self.npts == 0:  # filtrage tel qu'il n'y a plus de points
            QtGui.QMessageBox.about(None, "Error", "All points are filtered")

            self.mins = np.zeros(self.nmotors)
            self.maxs = np.ones(self.nmotors)
        else:
            self.mins = np.amin(self.data, axis=1)  # bornes pour chaque parametre
            self.maxs = np.amax(self.data, axis=1)


class Set3DparametersWindow(QtGui.QDialog):
    # definit une fenetre pour rentrer les parametres de dialogue de construction de map 3D
    def __init__(self, dataset, params, NX=1, NY=1, mins=None, maxs=None):
        self.params = params
        xdef = params.ix
        ydef = params.iy
        zdef = params.iz
        ilog = params.ilog
        io = params.io
        angle = params.angle
        bc = params.bc
        b1 = params.b1
        b2 = params.b2
        self.dataset = dataset
        self.NY = NY  # a priori le nombre de scans selectionne
        super(
            Set3DparametersWindow, self
        ).__init__()  # permet l'initialisation de la fenetre sans perdre les fonctions associees
        self.setWindowTitle("Parameters for 3D mesh computation")
        self.setFixedSize(QtCore.QSize(330, 230))
        self.mins = mins
        self.maxs = maxs

        """
        if mins==None:
            self.mins=np.zeros(dataset.nmotors)
        else:
            self.mins=mins

        if maxs==None:
            self.maxs=np.zeros(dataset.nmotors)
        else:
            self.maxs=maxs
        """
        # print self.mins,self.maxs
        self.lab1 = QtGui.QLabel(self)
        self.lab1.setGeometry(QtCore.QRect(35, 5, 100, 25))
        self.lab1.setText("component")

        self.lab2 = QtGui.QLabel(self)
        self.lab2.setGeometry(QtCore.QRect(140, 5, 55, 25))
        self.lab2.setText("bins")

        self.lab3 = QtGui.QLabel(self)
        self.lab3.setGeometry(QtCore.QRect(200, 5, 55, 25))
        self.lab3.setText("min")

        self.lab4 = QtGui.QLabel(self)
        self.lab4.setGeometry(QtCore.QRect(260, 5, 55, 25))
        self.lab4.setText("max")

        self.labx = QtGui.QLabel(self)
        self.labx.setGeometry(QtCore.QRect(5, 35, 25, 25))
        self.labx.setText("x")

        self.laby = QtGui.QLabel(self)
        self.laby.setGeometry(QtCore.QRect(5, 65, 25, 25))
        self.laby.setText("y")

        self.labz = QtGui.QLabel(self)
        self.labz.setGeometry(QtCore.QRect(5, 95, 25, 25))
        self.labz.setText("z")

        self.paramx = QtGui.QComboBox(self)
        self.paramx.setGeometry(QtCore.QRect(35, 35, 100, 25))
        self.paramx.addItems(dataset.nodenicknames)
        self.paramx.setCurrentIndex(xdef)

        self.paramy = QtGui.QComboBox(self)
        self.paramy.setGeometry(QtCore.QRect(35, 65, 100, 25))
        self.paramy.addItems(dataset.nodenicknames)
        self.paramy.addItem("None")
        self.paramy.setCurrentIndex(ydef)

        self.paramz = QtGui.QComboBox(self)
        self.paramz.setGeometry(QtCore.QRect(35, 95, 100, 25))
        self.paramz.addItems(dataset.nodenicknames)
        self.paramz.setCurrentIndex(zdef)

        self.binx = QtGui.QLineEdit(self)
        self.binx.setGeometry(QtCore.QRect(140, 35, 45, 25))
        self.binx.setText("%d" % (NX))

        self.biny = QtGui.QLineEdit(self)
        self.biny.setGeometry(QtCore.QRect(140, 65, 45, 25))
        self.biny.setText("%d" % (NY))

        self.zscale = QtGui.QComboBox(self)
        self.zscale.setGeometry(QtCore.QRect(140, 95, 100, 25))
        self.zscale.addItems(("z linear", "z log"))
        self.zscale.setCurrentIndex(ilog)

        self.xmin = QtGui.QLineEdit(self)
        self.xmin.setGeometry(QtCore.QRect(190, 35, 65, 25))

        self.ymin = QtGui.QLineEdit(self)
        self.ymin.setGeometry(QtCore.QRect(190, 65, 65, 25))

        self.xmax = QtGui.QLineEdit(self)
        self.xmax.setGeometry(QtCore.QRect(260, 35, 65, 25))

        self.ymax = QtGui.QLineEdit(self)
        self.ymax.setGeometry(QtCore.QRect(260, 65, 65, 25))

        self.changex(xdef)
        self.changey(ydef)

        self.io = QtGui.QRadioButton(self)
        self.io.setGeometry(QtCore.QRect(35, 125, 155, 25))
        self.io.setText("use orientation matrix")
        self.io.setChecked(io)

        self.angle = QtGui.QLineEdit(self)
        self.angle.setGeometry(QtCore.QRect(200, 125, 65, 25))
        self.angle.setText("%f" % (angle))

        self.bc = QtGui.QRadioButton(self)
        self.bc.setGeometry(QtCore.QRect(35, 155, 155, 25))
        self.bc.setText("correct background")
        self.bc.setChecked(bc)

        self.b1 = QtGui.QLineEdit(self)
        self.b1.setGeometry(QtCore.QRect(200, 155, 45, 25))
        self.b1.setText("%d" % (b1))

        self.b2 = QtGui.QLineEdit(self)
        self.b2.setGeometry(QtCore.QRect(250, 155, 45, 25))
        self.b2.setText("%d" % (b2))

        self.OK = QtGui.QPushButton(self)
        self.OK.setGeometry(QtCore.QRect(5, 185, 90, 25))
        self.OK.setText("OK")

        self.Cancel = QtGui.QPushButton(self)
        self.Cancel.setGeometry(QtCore.QRect(100, 185, 90, 25))
        self.Cancel.setText("Cancel")

        self.bgroup = QtGui.QButtonGroup(self)
        self.bgroup.setExclusive(False)
        self.bgroup.addButton(self.io)
        self.bgroup.addButton(self.bc)

        QtCore.QObject.connect(
            self.Cancel, QtCore.SIGNAL(_fromUtf8("clicked()")), self.closewin
        )
        QtCore.QObject.connect(
            self.OK, QtCore.SIGNAL(_fromUtf8("clicked()")), self.appl
        )
        QtCore.QObject.connect(
            self.paramx,
            QtCore.SIGNAL(_fromUtf8("currentIndexChanged (int)")),
            self.changex,
        )
        QtCore.QObject.connect(
            self.paramy,
            QtCore.SIGNAL(_fromUtf8("currentIndexChanged (int)")),
            self.changey,
        )
        self.exec_()

    def changex(self, i):
        # quand on change x, on change les valeurs min et max de x par defaut
        self.xmin.setText("%f" % (self.mins[i]))
        self.xmax.setText("%f" % (self.maxs[i]))

    def changey(self, i):
        # quand on change x, on change les valeurs min et max de x par defaut
        # sauf quand on est sur le dernier de la liste
        if i < len(self.mins):
            self.ymin.setText("%f" % (self.mins[i]))
            self.ymax.setText("%f" % (self.maxs[i]))
        else:
            self.ymin.setText("%d" % (1))
            self.ymax.setText("%d" % (self.NY))

    def appl(self):

        if self.parent:

            try:
                self.params.NX = int(self.binx.text())
                self.params.NY = int(self.biny.text())
                self.params.xmin = float(self.xmin.text())
                self.params.xmax = float(self.xmax.text())
                self.params.ymin = float(self.ymin.text())
                self.params.ymax = float(self.ymax.text())
                self.params.angle = float(self.angle.text())
                self.params.b1 = float(self.b1.text())
                self.params.b2 = float(self.b2.text())

            except Exception:
                QtGui.QMessageBox.about(self, "Error", "Input can only be a number")
                return

            ix = self.paramx.currentIndex()
            iy = self.paramy.currentIndex()
            iz = self.paramz.currentIndex()
            if ix == iy or iy == iz or ix == iy:
                QtGui.QMessageBox.about(self, "Error", "Select 3 different parameters")
                return

            self.params.ix = ix
            self.params.iy = iy
            self.params.iz = iz
            self.params.ilog = self.zscale.currentIndex()
            self.params.io = self.io.isChecked()
            self.params.bc = self.bc.isChecked()
        self.close()

    def closewin(self):
        self.close()


class SetFilterWindow(QtGui.QDialog):
    # definit une fenetre pour rentrer les parametres de dialogue de construction de map 3D
    def __init__(self, datafilter):

        super(
            SetFilterWindow, self
        ).__init__()  # permet l'initialisation de la fenetre sans perdre les fonctions associees
        self.datafilter = datafilter
        self.setWindowTitle("Filter Policy")
        self.setFixedSize(QtCore.QSize(200, 100))

        # print self.mins,self.maxs
        self.lab1 = QtGui.QLabel(self)
        self.lab1.setGeometry(QtCore.QRect(5, 5, 100, 25))
        self.lab1.setText("Threshold value")

        self.lab2 = QtGui.QLabel(self)
        self.lab2.setGeometry(QtCore.QRect(5, 35, 100, 25))
        self.lab2.setText("Filter policy")

        self.value = QtGui.QLineEdit(self)
        self.value.setGeometry(QtCore.QRect(105, 5, 90, 25))
        self.value.setText("%f" % (datafilter.value))

        self.role = QtGui.QComboBox(self)
        self.role.setGeometry(QtCore.QRect(105, 35, 90, 25))
        self.role.addItems(datafilter.roles)
        self.role.setCurrentIndex(datafilter.irole)

        self.OK = QtGui.QPushButton(self)
        self.OK.setGeometry(QtCore.QRect(5, 65, 90, 25))
        self.OK.setText("OK")

        self.Cancel = QtGui.QPushButton(self)
        self.Cancel.setGeometry(QtCore.QRect(100, 65, 90, 25))
        self.Cancel.setText("Cancel")

        QtCore.QObject.connect(
            self.Cancel, QtCore.SIGNAL(_fromUtf8("clicked()")), self.closewin
        )
        QtCore.QObject.connect(
            self.OK, QtCore.SIGNAL(_fromUtf8("clicked()")), self.appl
        )
        self.exec_()

    def appl(self):

        self.datafilter.irole = self.role.currentIndex()
        self.datafilter.role = self.datafilter.roles[self.datafilter.irole]

        try:
            self.datafilter.value = float(self.value.text())

        except Exception:
            QtGui.QMessageBox.about(self, "Error", "Input can only be a number")
            return

        self.close()

    def closewin(self):
        self.close()


class SetPrefWindow(QtGui.QDialog):
    # definit une fenetre pour rentrer les parametres d'affichage des preferences
    def __init__(self, pref):

        QtGui.QDialog.__init__(self)

        self.pref = pref
        self.setWindowTitle("Preferences")
        self.setFixedSize(QtCore.QSize(250, 160))

        self.lab1 = QtGui.QLabel(self)
        self.lab1.setGeometry(QtCore.QRect(5, 5, 90, 25))
        self.lab1.setText("Name display")

        self.nameDisplay = QtGui.QComboBox(self)
        self.nameDisplay.setGeometry(QtCore.QRect(95, 5, 150, 25))
        self.nameDisplay.addItems(pref.namedisplays)
        self.nameDisplay.setCurrentIndex(pref.inamedisplay)

        self.autoupdate = QtGui.QCheckBox(self)
        self.autoupdate.setGeometry(QtCore.QRect(5, 35, 150, 25))
        self.autoupdate.setText("auto-update fit")
        self.autoupdate.setChecked(pref.autoupdate)

        self.followscans = QtGui.QCheckBox(self)
        self.followscans.setGeometry(QtCore.QRect(5, 65, 150, 25))
        self.followscans.setText("follow fitted scans")
        self.followscans.setChecked(pref.followscans)

        self.OK = QtGui.QPushButton(self)
        self.OK.setGeometry(QtCore.QRect(5, 125, 90, 25))
        self.OK.setText("OK")

        self.Cancel = QtGui.QPushButton(self)
        self.Cancel.setGeometry(QtCore.QRect(100, 125, 90, 25))
        self.Cancel.setText("Cancel")

        QtCore.QObject.connect(
            self.Cancel, QtCore.SIGNAL(_fromUtf8("clicked()")), self.closewin
        )
        QtCore.QObject.connect(
            self.OK, QtCore.SIGNAL(_fromUtf8("clicked()")), self.appl
        )
        self.exec_()

    def appl(self):

        self.pref.inamedisplay = self.nameDisplay.currentIndex()
        self.pref.autoupdate = self.autoupdate.isChecked()
        self.pref.followscans = self.followscans.isChecked()

        self.close()

    def closewin(self):
        self.close()


class SelectMultiPointTool(InteractiveTool):
    TITLE = _("Point selection")
    ICON = "point_selection.png"
    MARKER_STYLE_SECT = "plot"
    MARKER_STYLE_KEY = "marker/curve"
    CURSOR = QtCore.Qt.PointingHandCursor

    def __init__(
        self,
        manager,
        mode="reuse",
        on_active_item=False,
        title=None,
        icon=None,
        tip=None,
        end_callback=None,
        toolbar_id=DefaultToolbarID,
        marker_style=None,
        switch_to_default_tool=None,
    ):
        super(SelectMultiPointTool, self).__init__(
            manager, toolbar_id, title=title, icon=icon, tip=tip
        )
        #                              switch_to_default_tool=switch_to_default_tool)
        assert mode in ("reuse", "create")
        self.mode = mode
        self.end_callback = end_callback
        self.marker = None
        self.last_pos = None
        self.on_active_item = on_active_item
        if marker_style is not None:
            self.marker_style_sect = marker_style[0]
            self.marker_style_key = marker_style[1]
        else:
            self.marker_style_sect = self.MARKER_STYLE_SECT
            self.marker_style_key = self.MARKER_STYLE_KEY
        self.impact = 0

    def set_marker_style(self, marker):
        marker.set_style(self.marker_style_sect, self.marker_style_key)

    def setup_filter(self, baseplot):
        filter = baseplot.filter
        # Initialisation du filtre
        start_state = filter.new_state()
        # Bouton gauche :
        handler = QtDragHandler(filter, QtCore.Qt.LeftButton, start_state=start_state)
        self.connect(handler, SIG_START_TRACKING, self.start)
        self.connect(handler, SIG_MOVE, self.move)
        self.connect(handler, SIG_STOP_NOT_MOVING, self.stop)
        self.connect(handler, SIG_STOP_MOVING, self.stop)
        return setup_standard_tool_filter(filter, start_state)

    def start(self, filter, event):
        if self.marker is None:
            title = ""
            if self.TITLE:
                title = "<b>%s</b><br>" % self.TITLE
            if self.on_active_item:
                # constraint_cb = filter.plot.on_active_curve
                # constraint_cb = self.on_active_curve
                constraint_cb = lambda x, y: self.on_active_curve(x, y, filter.plot)

                label_cb = lambda x, y: self.on_active_curve_label(x, y, filter.plot)
                """
                label_cb = lambda x, y: title + \
                           filter.plot.get_coordinates_str(x, y)
                """
            else:
                label_cb = lambda x, y: "%sx = %g<br>y = %g" % (title, x, y)
            self.marker = Marker(label_cb=label_cb, constraint_cb=constraint_cb)
            # print self.marker.xValue()   : 0
            self.set_marker_style(self.marker)
        self.marker.attach(filter.plot)
        self.marker.setZ(filter.plot.get_max_z() + 1)
        self.marker.setVisible(True)

    def stop(self, filter, event):
        self.move(filter, event)
        if self.mode != "reuse":
            self.marker.detach()
            self.marker = None
        if self.end_callback:
            self.end_callback(self)

    def move(self, filter, event):
        if self.marker is None:
            return  # something is wrong ...
        self.marker.move_local_point_to(0, event.pos())
        filter.plot.replot()
        self.last_pos = self.marker.xValue(), self.marker.yValue()

    def get_coordinates(self):
        return self.last_pos

    def on_active_curve(self, x, y, plot):
        curve = plot.get_last_active_item(ITrackableItemType)
        if curve:
            # x, y = get_closest_coordinates(x, y,curve)
            ax = curve.xAxis()
            ay = curve.yAxis()
            xc = plot.transform(ax, x)
            yc = plot.transform(ay, y)
            _distance, i, _inside, _other = curve.hit_test(QtCore.QPoint(xc, yc))
            x = curve.x(i)
            y = curve.y(i)
            self.impact = i
        return x, y

    def on_active_curve_label(self, x, y, plot):
        curve = plot.get_last_active_item(ITrackableItemType)
        if curve:
            label = "valeurs des parametres\n"
            for i in range(curve.dataset.nmotors):
                label = (
                    label
                    + curve.dataset.nodenicknames[i]
                    + ":%f\n" % (curve.dataset.data[i, self.impact])
                )
            return label


class Draw3DTool(CommandTool):
    def __init__(self, manager, toolbar_id=DefaultToolbarID):
        super(Draw3DTool, self).__init__(
            manager,
            _("Draw a 3D map"),
            get_icon("histogram2D.png"),
            toolbar_id=toolbar_id,
        )

    def activate_command(self, plot, checked):
        """Activate tool"""
        itemselected = plot.get_selected_items()
        itemselection = list(
            item for item in itemselected if ICurveItemType in item.types()
        )  # on ne prend que les courbes
        trueselection = list(item.dataset for item in itemselection)

        if len(trueselection) == 0:
            print("aucun curveitem selectionne")
            return

        Nitem = len(trueselection)
        item0 = itemselection[0]
        dataset0 = item0.dataset
        Nmotors = dataset0.nmotors

        mins = np.amin(
            np.concatenate(list(item.mins for item in trueselection)).reshape(
                Nitem, Nmotors
            ),
            axis=0,
        )
        maxs = np.amax(
            np.concatenate(list(item.maxs for item in trueselection)).reshape(
                Nitem, Nmotors
            ),
            axis=0,
        )

        xlabel = item0.xlabel
        zlabel = item0.ylabel
        names = dataset0.nodenicknames
        ix = names.index(xlabel)  # on propose par defaut ce qui est trace
        iz = names.index(zlabel)

        meshpar = MeshParameters(ix=ix, iy=0, iz=iz)
        Set3DparametersWindow(
            dataset0, meshpar, NX=item0.dataset.npts, NY=Nitem, mins=mins, maxs=maxs
        )  # ouvre une fenetre de dialogue
        prepare3Dmesh(trueselection, meshpar)


class ImageFileList(QtGui.QListWidget):
    """ A specialized QListWidget that displays the list
        of all nxs files in a given directory and its subdirectories. """

    def __init__(self, parent=None):
        QtGui.QListWidget.__init__(self, parent)
        self.setSelectionMode(3)
        # print "init"
        # item = QtGui.QListWidgetItem(self)
        # item.setText("vide")
        # self.setVisible(1)

    def setDirpath(self, dirpath):
        """ Set the current image directory and refresh the list. """
        self.dirpath = dirpath
        self.populate()

    def getnxs(self):
        """ Return a list of filenames of all
            supported images in self._dirpath. """

        self.nxs = [
            (name, os.path.join(root, name))
            for root, dirs, files in os.walk(self.dirpath)
            for name in files
            if name.endswith((".nxs"))
        ]
        self.nxs.sort()
        """
        self.nxs = [os.path.join(root, name)
          for root, dirs, files in os.walk(self._dirpath)
          for name in files
          if name.endswith((".nxs"))]
        """
        print(len(self.nxs), " files found")
        self.nxsdict = {a[0]: a[1] for a in self.nxs}
        # print self.nxsdict

    def populate(self):
        """ Fill the list with images from the
            current directory in self._dirpath. """

        # In case we're repopulating, clear the list
        self.clear()
        self.getnxs()

        # Create a list item for each image file,
        # setting the text and icon appropriately
        for names in self.nxs:
            shortname = names[0]
            item = QtGui.QListWidgetItem(self)
            item.setText(shortname)
            # item.setIcon(QtGui.QIcon(image))


def prepare3Dmesh(trueselection, meshpar):
    # prepare la mesh en fonction de la liste des scans et des parametres coches
    dataset = trueselection[0]

    xx = np.concatenate(list(item.data[meshpar.ix] for item in trueselection))
    xtitle = dataset.nodenicknames[meshpar.ix]

    if meshpar.iy < len(dataset.data):  # on prend la valeur y
        yy = np.concatenate(list(item.data[meshpar.iy] for item in trueselection))
        ytitle = dataset.nodenicknames[meshpar.iy]
    else:  # on prend le numero dans la liste, le premier scan commence a 1
        yy = np.zeros_like(dataset.data[meshpar.ix]) + 1
        # print len(yy)
        for i in range(1, len(trueselection)):
            ypp = np.zeros_like(trueselection[i].data[meshpar.ix]) + i + 1
            yy = np.concatenate((yy, ypp))
            # print len(yy)
        ytitle = "numero"

    if meshpar.bc:
        # sur chaque scan en z, on fait une correction d'intensite.
        b1 = int(meshpar.b1)
        b2 = int(meshpar.b1)
        if b1 > b2:
            b1, b2 = b2, b1
        b1 = max(0, b1)
        b2 = max(1, b2)
        b1 = min(b1, 99)
        b2 = min(b2, 100)
        fic = open("corr.txt", "w")
        for i in range(len(dataset.nodenicknames)):
            fic.write(dataset.nodenicknames[i])
            fic.write(" ")
        fic.write("background")
        fic.write("\n")
        vm = dict()
        for item in trueselection:
            z1 = np.percentile(item.data[meshpar.iz], b1)
            z2 = np.percentile(item.data[meshpar.iz], b2)
            vm[item] = np.ma.mean(np.ma.masked_outside(item.data[meshpar.iz], z1, z2))
            for i in range(len(dataset.nodenicknames)):
                fic.write("%f " % (item.data[i, 0]))
            fic.write("%f " % (vm[item]))
            fic.write("\n")
            vm[item] = 623 * np.arctan((item.data[15, 0] - 11.73) / 1.24)
            # print vm[item],item.data[:,0]
        zz = np.concatenate(
            list((item.data[meshpar.iz] - vm[item]) for item in trueselection)
        )

    else:
        zz = np.concatenate(list(item.data[meshpar.iz] for item in trueselection))

    bins = (meshpar.NY, meshpar.NX)
    bornes = [[meshpar.ymin, meshpar.ymax], [meshpar.xmin, meshpar.xmax]]

    scanlist = list(item.shortname for item in trueselection)
    scanlist.sort()  # on met en ordre la liste des scans et on prend le premier et le dernier
    ztitle = scanlist[0] + "->" + scanlist[-1]
    ztitle = ztitle + ":" + dataset.nodenicknames[meshpar.iz]

    if meshpar.io:
        # on utilise la matrice d'orientation pour tracer la map
        x2 = xx
        y2 = yy
        try:
            fichier = tables.openFile(dataset.longname)
            for node in fichier.listNodes("/")[0].SIXS:
                if "alpha_star" in node:
                    A_star = node.A_star.read()[0]
                    B_star = node.B_star.read()[0]
                    C_star = node.C_star.read()[0]
                    alpha_star = node.alpha_star.read()[0]
                    beta_star = node.beta_star.read()[0]
                    gamma_star = node.gamma_star.read()[0]
                    # print A_star,B_star,C_star,alpha_star,beta_star,gamma_star

        except Exception:
            print("la matrice d'orientation n'est pas definie")
        xtitle2 = xtitle
        ytitle2 = ytitle
        angle = 90.0  # valeurs par defaut
        unorm = 1.0
        vnorm = 1.0

        if xtitle == "h":
            if ytitle == "k":
                angle = gamma_star
                unorm = A_star
                vnorm = B_star
                xtitle2 = "kx"
                ytitle2 = "ky"
            elif ytitle == "l":
                angle = beta_star
                unorm = A_star
                vnorm = C_star
                xtitle2 = "kx"
                ytitle2 = "kz"
        elif xtitle == "k":
            if ytitle == "h":
                angle = -gamma_star
                unorm = B_star
                vnorm = A_star
                xtitle2 = "ky"
                ytitle2 = "kx"
            elif ytitle == "l":
                angle = alpha_star
                unorm = B_star
                vnorm = C_star
                xtitle2 = "ky"
                ytitle2 = "kz"
        elif xtitle == "l":
            if ytitle == "h":
                angle = -beta_star
                unorm = C_star
                vnorm = A_star
                xtitle2 = "kz"
                ytitle2 = "kx"
            elif ytitle == "k":
                angle = -alpha_star
                unorm = C_star
                vnorm = B_star
                xtitle2 = "kz"
                ytitle2 = "ky"
        angle = angle
        u = unorm * np.array([1, 0])
        v = vnorm * np.array([np.cos(angle * degtorad), np.sin(angle * degtorad)])
        x1 = xx * u[0] + yy * v[0]
        y1 = yy * u[1] + yy * v[1]
        angle2 = meshpar.angle

        cc = np.cos(angle2 * degtorad)
        ss = np.sin(angle2 * degtorad)
        x2 = x1 * cc - y1 * ss
        y2 = y1 * cc + x1 * ss
        u1 = [u[0] * cc - u[1] * ss, u[1] * cc + u[1] * ss]
        v1 = [v[0] * cc - v[1] * ss, v[1] * cc + v[1] * ss]

        bornes = [[np.amin(y2), np.amax(y2)], [np.amin(x2), np.amax(x2)]]
        # print bornes
        win = make3Dmesh(
            x2,
            y2,
            zz,
            zlog=meshpar.ilog,
            bins=bins,
            bornes=bornes,
            xtitle=xtitle2,
            ytitle=ytitle2,
            ztitle=ztitle,
        )
        plot = win.get_plot()
        eps = 1.0e-10
        if unorm == vnorm:
            if abs(angle - 90.0) < eps:
                sg = "p4"
            elif abs(angle - 60.0) < eps:
                sg = "p3"
            else:
                sg = "cm"
        else:
            if abs(angle - 90.0) < eps:
                sg = "p2"
            else:
                sg = "p1"
        shape = D2Dview.GridShape(O=[00, 000], u=u1, v=v1, sg=sg, order=2)
        shape.set_style("plot", "shape/gridshape")
        shape.setTitle("Reseau 1")
        shape.set_selectable(False)
        plot.add_item(shape)

    else:
        make3Dmesh(
            xx,
            yy,
            zz,
            zlog=meshpar.ilog,
            bins=bins,
            bornes=bornes,
            xtitle=xtitle,
            ytitle=ytitle,
            ztitle=ztitle,
        )


def make3Dmesh(
    x, y, z, zlog=0, bins=None, bornes=None, xtitle="x", ytitle="y", ztitle="3D mesh"
):
    # retourne un tableau 2D
    # print x.shape
    # print y.shape
    # print z.shape
    ones = np.ones(z.shape)

    if bins[0] > 1:  # s'il y a plus d'une ligne en y on fait une map
        # on change les bornes pour faire l'histogramme correctement:
        # par exemple si on a pour y 4 valeurs en 1 2 3 4 il faut prendre bornes=(0.5,4.5)
        bornesy = bornes[0]
        deltay = (bornesy[1] - bornesy[0]) / (bins[0] - 1)
        bornesy[0] = bornesy[0] - deltay / 2.0
        bornesy[1] = bornesy[1] + deltay / 2.0

        if zlog:
            H, yedges, xedges = np.histogram2d(
                y, x, bins=bins, range=bornes, weights=np.log(z + 1)
            )
        else:
            H, yedges, xedges = np.histogram2d(y, x, bins=bins, range=bornes, weights=z)

        C, yedges, xedges = np.histogram2d(y, x, bins=bins, range=bornes, weights=ones)
        # fic=open('hist.txt',"w")
        # for i in range(bins[0]):
        #    for j in range (bins[1]):
        #        fic.write("%f "%(H[i,j]))
        #    fic.write("\n")
        # Hc=np.where(C=00,0,H)   # inutile a priori si C=0 H=0

        ## Find indecies of zeros values
        # index = np.where(C==0)

        Cc = np.where(C == 0, 1, C)  # on met 1 pour eviter division par 0
        hist = H / Cc

        mask0 = np.isfinite(hist)

        hist[C == 0] = np.nan
        # histvalues=H[::-1] / Cc[::-1] #image inversee en hauteur
        # hist=make.image(histvalues,xdata=[xedges[0],xedges[-1]],ydata=[yedges[0],yedges[-1]])

        ## Create Boolean array of missing values
        mask = np.isfinite(hist)

        # interpolation des donnees manquantes
        values = hist[mask].flatten()

        ## Find indecies of finite values
        index = np.where(mask == True)

        """
      x0,y0 = index[0],index[1]
      print "valeurs grille non masque"
      print x
      print y
      """
        grid = np.where(mask0 == True)

        ## Grid irregular points to regular grid using delaunay triangulation
        values = griddata(index, values, grid, method="linear").reshape(hist.shape)
        # a la fin, il reste des NAN, si on veut pouvoir ajuster le contraste avec guiqwt, il
        # faut changer True en False dans le module guiqwt.image, get_histogram ligne 672

        win = D2Dview.D2DDialog()
        win.setWindowTitle("3D mesh")
        win.set_data(
            values,
            xdata=[xedges[0], xedges[-1]],
            ydata=[yedges[0], yedges[-1]],
            colormap="jet",
        )
        win.show_image()
        plot = win.get_plot()
        plot.set_axis_title("left", ytitle)
        plot.set_axis_title("bottom", xtitle)
        plot.set_title(ztitle)
        plot.set_axis_direction("left", reverse=False)

    else:  # on trace un scan moyen
        if zlog:
            H, xedges = np.histogram(
                x, bins=bins[1], range=bornes[1], weights=np.log(z + 1)
            )
        else:
            H, xedges = np.histogram(x, bins=bins[1], range=bornes[1], weights=z)

        C, xedges = np.histogram(x, bins=bins[1], range=bornes[1], weights=ones)

        win = CurveDialog(wintitle="Scan moyen")
        plot = win.get_plot()
        plot.set_axis_title("left", ztitle)
        plot.set_axis_title("bottom", xtitle)
        Cc = np.where(C == 0, 1, C)  # on met 1 pour eviter division par 0
        hist = H / Cc
        hist[C == 0] = np.nan  # on met nan la ou il n'y a pas de valeur
        xedges = xedges - (xedges[1] - xedges[0]) / 2.0  # on decale d'un demi pas
        xedges = xedges[1:]
        plot.add_item(make.curve(xedges, hist))
        win.show()
        """

    win.image=hist
    win.setGeometry(QtCore.QRect(420,50,800, 600))
    plot=win.get_plot()
    plot.add_item(hist)
    plot.set_axis_direction("left", reverse=False)
    plot.set_axis_title("left",ytitle)
    plot.set_axis_title("bottom",xtitle)
    plot.font_title.setPointSize (9)
    plot.set_title(ztitle)
    plot.set_aspect_ratio(lock=False)
    win.show()

    """
    """
    extent = [xedges[0], xedges[-1], yedges[0], yedges[-1]]

    #pylab.imshow(H[::-1] / C[::-1], extent=extent, interpolation='nearest', aspect=aspect)
    pylab.imshow(H[::-1] / Cc[::-1], extent=extent, interpolation='nearest')
    pylab.xlim(xedges[0], xedges[-1])
    pylab.ylim(yedges[0], yedges[-1])
    pylab.clim()
    pylab.show()
    """
    return win


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName(_fromUtf8("MainWindow"))
        MainWindow.setGeometry(QtCore.QRect(10, 50, 420, 600))
        MainWindow.setWindowTitle(
            QtGui.QApplication.translate(
                "MainWindow", "nxsViewer", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.centralwidget = QtGui.QWidget(MainWindow)
        self.centralwidget.setObjectName(_fromUtf8("centralwidget"))

        self.maxpar = 30  # probleme de maxpar a gerer...

        self.initlist()
        defaultfont = QtGui.QFont()
        defaultfont.setPointSize(10)
        QtGui.QApplication.setFont(defaultfont)
        self.listfiles = ImageFileList(parent=self.centralwidget)
        self.listfiles.setGeometry(QtCore.QRect(10, 10, 250, 550))
        self.listfiles.setObjectName(_fromUtf8("listfiles"))

        self.autoadd = QtGui.QRadioButton(self.centralwidget)
        self.autoadd.setGeometry(QtCore.QRect(270, 10, 50, 30))
        self.autoadd.setText("add")
        self.autoadd.setFont(defaultfont)

        self.autoreplace = QtGui.QRadioButton(self.centralwidget)
        self.autoreplace.setGeometry(QtCore.QRect(320, 10, 80, 30))
        self.autoreplace.setText("replace")

        self.Xboxlabel = QtGui.QLabel(self.centralwidget)
        self.Xboxlabel.setGeometry(QtCore.QRect(270, 30, 30, 30))
        self.Xboxlabel.setObjectName(_fromUtf8("Xboxlabel"))
        self.Xboxlabel.setText("X")

        self.Yboxlabel = QtGui.QLabel(self.centralwidget)
        self.Yboxlabel.setGeometry(QtCore.QRect(290, 30, 30, 30))
        self.Yboxlabel.setObjectName(_fromUtf8("Xboxlabel"))
        self.Yboxlabel.setText("Y")

        self.Filterboxlabel = QtGui.QLabel(self.centralwidget)
        self.Filterboxlabel.setGeometry(QtCore.QRect(310, 30, 30, 30))
        self.Filterboxlabel.setObjectName(_fromUtf8("Filterboxlabel"))
        self.Filterboxlabel.setText("Filter")
        """
        self.Zboxlabel = QtGui.QLabel(self.centralwidget)
        self.Zboxlabel.setGeometry(QtCore.QRect(330, 10, 30, 30))
        self.Zboxlabel.setObjectName(_fromUtf8("Xboxlabel"))
        self.Zboxlabel.setText("Z")
        """

        self.XGroup = QtGui.QButtonGroup(self.centralwidget)
        self.XCheckBoxes = list()
        self.YCheckBoxes = list()
        self.FilterGroup = QtGui.QButtonGroup(self.centralwidget)
        self.FilterCheckBoxes = list()

        v = 60

        for param in range(self.maxpar):
            cbox = QtGui.QCheckBox(self.centralwidget)
            cbox.hide()
            cbox.setGeometry(QtCore.QRect(270, v, 20, 20))
            self.XCheckBoxes.append(cbox)
            self.XGroup.addButton(cbox)
            QtCore.QObject.connect(
                cbox,
                QtCore.SIGNAL(_fromUtf8("stateChanged (int)")),
                self.boxselection_changed,
            )

            cbox = QtGui.QCheckBox(self.centralwidget)
            cbox.hide()
            cbox.setGeometry(QtCore.QRect(290, v, 20, 20))
            self.YCheckBoxes.append(cbox)
            QtCore.QObject.connect(
                cbox,
                QtCore.SIGNAL(_fromUtf8("stateChanged (int)")),
                self.boxselection_changed,
            )

            cbox = QtGui.QCheckBox(self.centralwidget)
            cbox.hide()
            cbox.setGeometry(QtCore.QRect(310, v, 300, 20))
            self.FilterCheckBoxes.append(cbox)
            self.FilterGroup.addButton(cbox)
            QtCore.QObject.connect(
                cbox,
                QtCore.SIGNAL(_fromUtf8("stateChanged (int)")),
                self.boxselection_changed,
            )

            v = v + 20
        self.nCheckBoxes = 0  # au debut, pas de cases affichees
        """
        self.XCheckBoxes[self.maxpar-1].setChecked(1)
        self.XCheckBoxes[7].setChecked(1)
        self.YCheckBoxes[0].setChecked(1)
        """

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtGui.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 400, 20))
        self.menubar.setObjectName(_fromUtf8("menubar"))

        self.menuFile = QtGui.QMenu(self.menubar)
        self.menuFile.setTitle(
            QtGui.QApplication.translate(
                "MainWindow", "File", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.menuFile.setObjectName(_fromUtf8("menuFile"))

        self.menuEdit = QtGui.QMenu(self.menubar)
        self.menuEdit.setTitle(
            QtGui.QApplication.translate(
                "MainWindow", "Edit", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.menuEdit.setObjectName(_fromUtf8("menuEdit"))

        self.menuData = QtGui.QMenu(self.menubar)
        self.menuData.setTitle(
            QtGui.QApplication.translate(
                "MainWindow", "Data", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.menuData.setObjectName(_fromUtf8("menuData"))

        self.menuGraph = QtGui.QMenu(self.menubar)
        self.menuGraph.setTitle(
            QtGui.QApplication.translate(
                "MainWindow", "Graph", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.menuGraph.setObjectName(_fromUtf8("menuEdit"))

        MainWindow.setMenuBar(self.menubar)

        self.statusbar = QtGui.QStatusBar(MainWindow)
        self.statusbar.setObjectName(_fromUtf8("statusbar"))
        MainWindow.setStatusBar(self.statusbar)

        self.actionOpen = QtGui.QAction(MainWindow)
        self.actionOpen.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Open", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionOpen.setObjectName(_fromUtf8("actionImage"))
        self.actionSave = QtGui.QAction(MainWindow)
        self.actionSave.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Save", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionSave.setObjectName(_fromUtf8("actionImage"))
        self.actionSave.setDisabled(1)
        self.actionQuit = QtGui.QAction(MainWindow)
        self.actionQuit.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Quit", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionQuit.setObjectName(_fromUtf8("actionQuit"))

        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addAction(self.actionSave)
        self.menuFile.addAction(self.actionQuit)

        self.actionCopy = QtGui.QAction(MainWindow)
        self.actionCopy.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Copy", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionCopy.setObjectName(_fromUtf8("actionCopy"))
        self.actionClear = QtGui.QAction(MainWindow)
        self.actionClear.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Clear", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionClear.setObjectName(_fromUtf8("actionClear"))

        self.menuEdit.addAction(self.actionCopy)
        self.menuEdit.addAction(self.actionClear)

        self.actionNewWindow = QtGui.QAction(MainWindow)
        self.actionNewWindow.setText(
            QtGui.QApplication.translate(
                "MainWindow", "New window", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionNewWindow.setObjectName(_fromUtf8("NewWindow "))

        self.actionXLog = QtGui.QAction(MainWindow)
        self.actionXLog.setText(
            QtGui.QApplication.translate(
                "MainWindow", "X log", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionXLog.setObjectName(_fromUtf8("XLog"))
        self.actionXLog.setCheckable(1)

        self.actionYLog = QtGui.QAction(MainWindow)
        self.actionYLog.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Y log", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionYLog.setObjectName(_fromUtf8("YLog"))
        self.actionYLog.setCheckable(1)

        self.actionAutoscale = QtGui.QAction(MainWindow)
        self.actionAutoscale.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Autoscale", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionAutoscale.setObjectName(_fromUtf8("Autoscale"))
        self.actionAutoscale.setCheckable(1)

        self.menuGraph.addAction(self.actionNewWindow)
        self.menuGraph.addAction(self.actionXLog)
        self.menuGraph.addAction(self.actionYLog)
        self.menuGraph.addAction(self.actionAutoscale)

        self.actionFilter = QtGui.QAction(MainWindow)
        self.actionFilter.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Filter policy", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionFilter.setObjectName(_fromUtf8("InitFilter "))

        self.actionInitFit = QtGui.QAction(MainWindow)
        self.actionInitFit.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Fit", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionInitFit.setObjectName(_fromUtf8("InitFit "))

        self.action3DMesh = QtGui.QAction(MainWindow)
        self.action3DMesh.setText(
            QtGui.QApplication.translate(
                "MainWindow", "3DMesh", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.action3DMesh.setObjectName(_fromUtf8("Draw a 3D Mesh "))

        self.actionPreferences = QtGui.QAction(MainWindow)
        self.actionPreferences.setText(
            QtGui.QApplication.translate(
                "MainWindow", "Preferences", None, QtGui.QApplication.UnicodeUTF8
            )
        )
        self.actionPreferences.setObjectName(_fromUtf8("Preferences "))

        self.menuData.addAction(self.actionFilter)
        self.menuData.addAction(self.actionInitFit)
        self.menuData.addAction(self.action3DMesh)
        self.menuData.addAction(self.actionPreferences)

        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuEdit.menuAction())
        self.menubar.addAction(self.menuData.menuAction())
        self.menubar.addAction(self.menuGraph.menuAction())
        self.makewindow()
        self.retranslateUi(MainWindow)

        QtCore.QObject.connect(
            self.actionOpen,
            QtCore.SIGNAL(_fromUtf8("triggered()")),
            self.ouvrirFichiers,
        )
        QtCore.QObject.connect(
            self.actionSave, QtCore.SIGNAL(_fromUtf8("triggered()")), self.enregistrer
        )
        QtCore.QObject.connect(
            self.actionQuit, QtCore.SIGNAL(_fromUtf8("triggered()")), self.fermer
        )

        QtCore.QObject.connect(
            self.actionXLog, QtCore.SIGNAL(_fromUtf8("triggered()")), self.setXLog
        )
        QtCore.QObject.connect(
            self.actionYLog, QtCore.SIGNAL(_fromUtf8("triggered()")), self.setYLog
        )
        QtCore.QObject.connect(
            self.actionNewWindow,
            QtCore.SIGNAL(_fromUtf8("triggered()")),
            self.makewindow,
        )

        QtCore.QObject.connect(
            self.actionFilter, QtCore.SIGNAL(_fromUtf8("triggered()")), self.setfilter
        )
        QtCore.QObject.connect(
            self.actionInitFit, QtCore.SIGNAL(_fromUtf8("triggered()")), self.initfit
        )
        QtCore.QObject.connect(
            self.action3DMesh, QtCore.SIGNAL(_fromUtf8("triggered()")), self.draw3Dmesh
        )
        QtCore.QObject.connect(
            self.actionPreferences,
            QtCore.SIGNAL(_fromUtf8("triggered()")),
            self.setpreferences,
        )

        QtCore.QObject.connect(
            self.listfiles,
            QtCore.SIGNAL(_fromUtf8("itemSelectionChanged ()")),
            self.item_selectionchanged,
        )
        QtCore.QObject.connect(
            self.listfiles,
            QtCore.SIGNAL(_fromUtf8("itemClicked (QListWidgetItem *)")),
            self.item_clicked,
        )

        QtCore.QMetaObject.connectSlotsByName(MainWindow)
        self.clipboard = QtGui.QApplication.clipboard()

        # self.Image.setFocus()
        self.csuite = (
            "#00bfff",
            "#ff6a6a",
            "#98fb98",
            "#ffd700",
            "#ee82e2",
            "#e9967a",
            "#4169e1",
            "#ff0000",
            "#00ff00",
            "#ffa500",
        )
        self.icolor = 0
        self.listtitles = list()
        direct = "E:\Geoffroy-Boulot\Au-Cu\Sixs-decembre2013"
        self.listfiles.setDirpath(direct)
        self.autoreplace.setChecked(1)
        self.isafit = 0
        self.lastitem = None
        self.nXBoxesChecked = 0
        self.nYBoxesChecked = 0
        self.datafilter = DataFilter()
        self.pref = PrefParameters()
        self.meshpar = MeshParameters()

    def refreshboxes(self, dataset):
        # on met à jour la liste des cases a cocher en fonction du dataset selectionne
        if dataset.nmotors > self.nCheckBoxes:  # on rajoute des cases
            for i in range(self.nCheckBoxes, dataset.nmotors):
                self.XCheckBoxes[i].show()
                self.YCheckBoxes[i].show()
                self.FilterCheckBoxes[i].show()

        elif dataset.nmotors < self.nCheckBoxes:  # on rajoute des cases
            for i in range(dataset.nmotors, self.nCheckBoxes):
                self.XCheckBoxes[i].hide()
                self.YCheckBoxes[i].hide()
                self.FilterCheckBoxes[i].hide()

                if self.XCheckBoxes[i].isChecked():
                    self.XCheckBoxes[i].setChecked(0)
                if self.FilterCheckBoxes[i].isChecked():
                    self.FilterCheckBoxes[i].setChecked(0)
                """
                a priori pas necessaire en Y, comme ca on garde en memoire les cases cochees
                if self.YCheckBoxes[i].isChecked():
                    self.YCheckBoxes[i].setChecked(0)
                """
        self.nXBoxesChecked = 0
        self.nYBoxesChecked = 0

        for i in range(dataset.nmotors):
            self.FilterCheckBoxes[i].setText(dataset.nodenicknames[i])
            if self.XCheckBoxes[i].isChecked():
                self.nXBoxesChecked = self.nXBoxesChecked + 1
            if self.YCheckBoxes[i].isChecked():
                self.nYBoxesChecked = self.nYBoxesChecked + 1

        self.nCheckBoxes = dataset.nmotors

    def initfit(self):
        # ici on inclue une figure de fit provenant de grafit
        if self.isafit == 0:
            from grafit2.grafit2 import Ui_FitWindow

            #           import grafit2 as grafit
            self.figfit = Ui_FitWindow()
            #           self.figfit=grafit.Ui_FitWindow()
            self.figfit.setupUi()
            self.figfit.setupdatestart(self.pref.autoupdate)
            # tags est une liste de couples [non,valeur]
            self.figfit.settags([], saveint=True)

            self.isafit = 1
            # ici on redirige le signal de fermeture de la fenetre de fit
            self.figfit.closeEvent = self.closefit

        self.figfit.move(10, 500)
        self.figfit.show()
        self.updatefit()

    def closefit(self, closeevent):
        self.isafit = 0

    def showfit(self, itemselected):
        x, y = itemselected.get_data()
        xlabel = itemselected.xlabel
        ylabel = itemselected.ylabel
        xmin, xmax = self.plot.get_axis_limits("bottom")

        y2 = np.compress((x >= xmin) & (x <= xmax), y)
        x2 = np.compress((x >= xmin) & (x <= xmax), x)

        # on recupere les donnees du scan (angles, H,K,L) pour les envoyer a grafit
        try:
            fichier = tables.openFile(itemselected.dataset.longname)
            group = fichier.listNodes("/")[0]
            leafnames = [
                "I14-C-CX2__EX__DIFF-UHV-H__#1/raw_value",
                "I14-C-CX2__EX__DIFF-UHV-K__#1/raw_value",
                "I14-C-CX2__EX__DIFF-UHV-L__#1/raw_value",
                "SIXS/I14-C-CX2__EX__mu-uhv__#1/raw_value",
                "SIXS/I14-C-CX2__EX__omega-uhv__#1/raw_value",
                "SIXS/I14-C-CX2__EX__delta-uhv__#1/raw_value",
                "SIXS/I14-C-CX2__EX__gamma-uhv__#1/raw_value",
            ]
            leafshortnames = ["H", "K", "L", "mu", "omega", "delta", "gamma"]
            leafs = [group._f_getChild(leafname) for leafname in leafnames]
            leafvalues = list(leaf.read()[0] for leaf in leafs)
            tags = zip(leafshortnames, leafvalues)

        except Exception:
            tags = []

        self.figfit.setvalexp(
            x2,
            y2,
            xlabel=xlabel,
            ylabel=ylabel,
            title=itemselected.curveparam.label,
            xlog=self.actionXLog.isChecked(),
            ylog=self.actionYLog.isChecked(),
            tags=tags,
            followscans=self.pref.followscans,
        )
        self.figfit.show()

    def updatefit(self):
        # print "update fit"

        itemselected = self.plot.get_active_item(force=False)

        if itemselected == None:
            itemselected = self.lastitem

        if itemselected == None:
            pass

        elif ICurveItemType in itemselected.types():
            # on a selectionne une curve
            # print "curve"
            if self.isafit:
                self.showfit(itemselected)

    def initlist(self):
        self.paramlist = [
            "y",
            "ybrut",
            "filters",
            "mu",
            " ",
            "delta",
            "gamma",
            "h",
            "k",
            "l",
            "q",
        ]

    def fermer(self):
        if self.isafit:
            self.figfit.close()
        self.win.close()
        MainWindow.close()

    def enregistrer(self):
        pass

    def setXLog(self):
        if self.actionXLog.isChecked():
            self.plot.set_axis_scale("bottom", "log")
        else:
            self.plot.set_axis_scale("bottom", "lin")
        self.plot.show_items()

    def setYLog(self):
        if self.actionYLog.isChecked():
            self.plot.set_axis_scale("left", "log")
        else:
            self.plot.set_axis_scale("left", "lin")
        self.plot.show_items()

    def changeZ(self):
        pass

    def retranslateUi(self, MainWindow):
        pass

    def drawcurve(self, dataset):
        kid = self.XGroup.checkedId()  # a priori il y  a toujours un X de selectionne
        if kid == -1:
            print("no X selected")
        else:
            ix = -kid - 2
            absi = dataset.data[ix]
        for iy in range(dataset.nmotors):
            if self.YCheckBoxes[iy].isChecked():
                # print i,params[i]
                xlabel = dataset.nodenicknames[ix]
                ylabel = dataset.nodenicknames[iy]

                title = dataset.shortname + "_" + ylabel + "(" + xlabel + ")"

                if (
                    title not in self.listtitles
                ):  # on ne retrace pas des courbes deja faites

                    self.lastitem = self.drawgraph(
                        absi,
                        dataset.data[iy],
                        title=title,
                        xlabel=xlabel,
                        ylabel=ylabel,
                    )
                    self.lastitem.dataset = dataset
                    self.plot.set_titles(xlabel=xlabel, ylabel=ylabel)

                    self.addplot(self.lastitem)
                    self.icolor = (self.icolor + 1) % 10
                    self.listtitles.append(title)

    def draw3Dmesh(self):
        trueselection = list()
        scanlist = list()
        for item in self.listfiles.selectedItems():
            shortname = str(item.text())
            longname = self.listfiles.nxsdict[shortname]
            dataset = DataSet(
                longname, shortname, datafilter=self.datafilter, pref=self.pref
            )
            trueselection.append(dataset)
            scanlist.append(shortname)

        if len(trueselection) == 0:
            print("aucun item selectionne")
            return

        Nitem = len(trueselection)
        Nmotors = dataset.nmotors

        mins = np.amin(
            np.concatenate(list(dataset.mins for dataset in trueselection)).reshape(
                Nitem, Nmotors
            ),
            axis=0,
        )
        maxs = np.amax(
            np.concatenate(list(dataset.maxs for dataset in trueselection)).reshape(
                Nitem, Nmotors
            ),
            axis=0,
        )
        Set3DparametersWindow(
            dataset, self.meshpar, NX=dataset.npts, NY=Nitem, mins=mins, maxs=maxs
        )  # ouvre une fenetre de dialogue
        prepare3Dmesh(trueselection, self.meshpar)

    def makewindow(self):
        self.win = CurveDialog(
            edit=False,
            toolbar=True,
            wintitle="Scan window",
            options=dict(xlabel="xlabel", ylabel="ylabel"),
        )
        self.win.setGeometry(QtCore.QRect(440, 50, 800, 600))
        self.plot = self.win.get_plot()
        self.win.get_itemlist_panel().show()
        self.plot.set_items_readonly(False)
        self.win.add_tool(
            SelectMultiPointTool, title=None, on_active_item=True, mode="create"
        )
        self.win.add_tool(Draw3DTool)
        self.win.show()
        self.addplot(make.legend())
        self.plot.connect(self.plot, SIG_ACTIVE_ITEM_CHANGED, self.updatefit)
        # self.win.exec_()

    def addplot(self, *items):

        for item in items:
            self.plot.add_item(item)
        # self.plot.activateWindow() met la fenêtre self.win au premier plan
        if self.actionAutoscale.isChecked():
            self.plot.do_autoscale(replot=True)
        self.plot.show_items()

    def drawgraph(self, absi, ordo, title=None, xlabel=None, ylabel=None):

        """i
        #inutile on a mis des filtres
        if self.actionYLog.isChecked():
            ymin=min(ordo)
            #print ymin
            if ymin<=0.0001:
                ymax=max(ordo)+1
                ycorr=np.where(ordo <= 0.0001, ymax+1, ordo)
                ymin=min(ycorr)
                ordo=np.where(ycorr > ymax, ymin, ycorr)
        """
        item = make.curve(absi, ordo, title=title, color=self.csuite[self.icolor])
        item.xlabel = xlabel
        item.ylabel = ylabel

        return item

    def ouvrirFichiers(self):

        self.dirfiles = QtGui.QFileDialog.getExistingDirectory(None, "Browse Directory")
        print(str(self.dirfiles))
        self.listfiles.setDirpath(str(self.dirfiles))

    def item_selectionchanged(self):
        # self.item_clicked(self.listfiles.selectedItems()[0])

        if self.autoreplace.isChecked():  # on efface
            self.icolor = 0
            self.listtitles = list()
            self.plot.del_all_items()
            self.addplot(make.legend())

        if (
            self.nXBoxesChecked > 0 and self.nYBoxesChecked > 0
        ):  # pas la peine de perdre du temps si rien n'est coche
            for item in self.listfiles.selectedItems():  # on ajoute les items
                self.init_item(item)
            if self.pref.autoupdate:
                self.updatefit()  # on update le fit avec le dernier item si self.pref.autoupdate est True

    def item_clicked(self, item):
        # le fichier a lire est obtenu par concatenation du directory et du filename

        self.shortname = str(item.text())
        longname = self.listfiles.nxsdict[self.shortname]

        self.currentdataset = DataSet(
            longname, self.shortname, datafilter=self.datafilter, pref=self.pref
        )
        self.refreshboxes(self.currentdataset)

    def init_item(self, item):
        self.shortname = str(item.text())
        longname = self.listfiles.nxsdict[self.shortname]

        self.currentdataset = DataSet(
            longname, self.shortname, datafilter=self.datafilter, pref=self.pref
        )
        self.refreshboxes(self.currentdataset)
        self.isacurve = 1
        self.drawcurve(self.currentdataset)

    def boxselection_changed(self, state):

        self.nXBoxesChecked = 0
        self.nYBoxesChecked = 0

        kid = self.FilterGroup.checkedId()
        self.datafilter.ifil = -kid - 2

        for i in range(self.nCheckBoxes):
            if self.XCheckBoxes[i].isChecked():
                self.nXBoxesChecked = self.nXBoxesChecked + 1
            if self.YCheckBoxes[i].isChecked():
                self.nYBoxesChecked = self.nYBoxesChecked + 1
        self.item_selectionchanged()

    def setfilter(self):
        # ouvre une fenetre pour le reglage des filtres
        SetFilterWindow(self.datafilter)

    def setpreferences(self):
        # ouvre une fenetre pour le reglage des filtres
        SetPrefWindow(self.pref)
        if self.isafit:
            self.figfit.setupdatestart(self.pref.autoupdate)


if __name__ == "__main__":
    import sys

    app = QtGui.QApplication(sys.argv)
    MainWindow = QtGui.QMainWindow()
    ui = Ui_MainWindow()
    ui.setupUi(MainWindow)
    ui.listfiles.setDirpath(os.getcwd())
    MainWindow.show()
    sys.exit(app.exec_())
